@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2020 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2021 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -17,9 +17,9 @@
 
 import sys
 
-if sys.hexversion < 0x03050000:
-    print("Sorry, requires Python 3.5 or above")
-    print("You can read more at: https://sabnzbd.org/python3")
+if sys.hexversion < 0x03060000:
+    print("Sorry, requires Python 3.6 or above")
+    print("You can read more at: https://sabnzbd.org/wiki/installation/install-off-modules")
     sys.exit(1)
 
 import logging
@@ -34,6 +34,8 @@ import subprocess
 import ssl
 import time
 import re
+import gc
+from typing import List, Dict, Any
 
 try:
     import Cheetah
@@ -46,7 +48,7 @@ try:
 except ImportError as e:
     print("Not all required Python modules are available, please check requirements.txt")
     print("Missing module:", e.name)
-    print("You can read more at: https://sabnzbd.org/python3")
+    print("You can read more at: https://sabnzbd.org/wiki/installation/install-off-modules")
     print("If you still experience problems, remove all .pyc files in this folder and subfolders")
     sys.exit(1)
 
@@ -66,15 +68,18 @@ from sabnzbd.misc import (
     get_serv_parms,
     get_from_url,
     upload_file_to_sabnzbd,
+    is_localhost,
+    is_lan_addr,
 )
 from sabnzbd.filesystem import get_ext, real_path, long_path, globber_full, remove_file
 from sabnzbd.panic import panic_tmpl, panic_port, panic_host, panic, launch_a_browser
-import sabnzbd.scheduler as scheduler
 import sabnzbd.config as config
 import sabnzbd.cfg
 import sabnzbd.downloader
 import sabnzbd.notifier as notifier
 import sabnzbd.zconfig
+from sabnzbd.getipaddress import localipv4, publicipv4, ipv6
+import sabnzbd.utils.ssdp as ssdp
 
 try:
     import win32api
@@ -102,36 +107,62 @@ def guard_loglevel():
     LOG_FLAG = True
 
 
+def warning_helpful(*args, **kwargs):
+    """ Wrapper to ignore helpfull warnings if desired """
+    if sabnzbd.cfg.helpfull_warnings():
+        return logging.warning(*args, **kwargs)
+    return logging.info(*args, **kwargs)
+
+
+logging.warning_helpful = warning_helpful
+
+
 class GUIHandler(logging.Handler):
-    """ Logging handler collects the last warnings/errors/exceptions
-        to be displayed in the web-gui
+    """Logging handler collects the last warnings/errors/exceptions
+    to be displayed in the web-gui
     """
 
     def __init__(self, size):
         """ Initializes the handler """
         logging.Handler.__init__(self)
-        self.size = size
-        self.store = []
+        self._size: int = size
+        self.store: List[Dict[str, Any]] = []
 
-    def emit(self, record):
+    def emit(self, record: logging.LogRecord):
         """ Emit a record by adding it to our private queue """
-        if record.levelname == "WARNING":
-            sabnzbd.LAST_WARNING = record.msg % record.args
-        else:
-            sabnzbd.LAST_ERROR = record.msg % record.args
-
-        if len(self.store) >= self.size:
-            # Loose the oldest record
-            self.store.pop(0)
+        # If % is part of the msg, this could fail
         try:
-            # Append traceback, if available
-            warning = {"type": record.levelname, "text": record.msg % record.args, "time": int(time.time())}
-            if record.exc_info:
-                warning["text"] = "%s\n%s" % (warning["text"], traceback.format_exc())
-            self.store.append(warning)
-        except UnicodeDecodeError:
-            # Catch elusive Unicode conversion problems
-            pass
+            parsed_msg = record.msg % record.args
+        except TypeError:
+            parsed_msg = record.msg + str(record.args)
+
+        warning = {
+            "type": record.levelname,
+            "text": parsed_msg,
+            "time": int(time.time()),
+            "origin": "%s%d" % (record.filename, record.lineno),
+        }
+
+        # Append traceback, if available
+        if record.exc_info:
+            warning["text"] = "%s\n%s" % (warning["text"], traceback.format_exc())
+
+        # Do not notify the same notification within 1 minute from the same source
+        # This prevents endless looping if the notification service itself throws an error/warning
+        # We don't check based on message content, because if it includes a timestamp it's not unique
+        if not any(
+            stored_warning["origin"] == warning["origin"] and stored_warning["time"] + DEF_TIMEOUT > time.time()
+            for stored_warning in self.store
+        ):
+            if record.levelno == logging.WARNING:
+                sabnzbd.notifier.send_notification(T("Warning"), parsed_msg, "warning")
+            else:
+                sabnzbd.notifier.send_notification(T("Error"), parsed_msg, "error")
+
+        # Loose the oldest record
+        if len(self.store) >= self._size:
+            self.store.pop(0)
+        self.store.append(warning)
 
     def clear(self):
         self.store = []
@@ -178,6 +209,7 @@ def print_help():
     print("      --no-login           Start with username and password reset")
     print("      --log-all            Log all article handling (for developers)")
     print("      --disable-file-log   Logging is only written to console")
+    print("      --console            Force logging to console")
     print("      --new                Run a new instance of SABnzbd")
     print()
     print("NZB (or related) file:")
@@ -191,7 +223,7 @@ def print_version():
             """
 %s-%s
 
-Copyright (C) 2007-2020 The SABnzbd-Team <team@sabnzbd.org>
+Copyright (C) 2007-2021 The SABnzbd-Team <team@sabnzbd.org>
 SABnzbd comes with ABSOLUTELY NO WARRANTY.
 This is free software, and you are welcome to redistribute it
 under certain conditions. It is licensed under the
@@ -233,7 +265,7 @@ def daemonize():
 
     # Get log file  path and remove the log file if it got too large
     log_path = os.path.join(sabnzbd.cfg.log_dir.get_path(), DEF_LOG_ERRFILE)
-    if os.path.exists(log_path) and os.path.getsize(log_path) > sabnzbd.cfg.log_size.get_int():
+    if os.path.exists(log_path) and os.path.getsize(log_path) > sabnzbd.cfg.log_size():
         remove_file(log_path)
 
     # Replace file descriptors for stdin, stdout, and stderr
@@ -276,7 +308,7 @@ def identify_web_template(key, defweb, wdir):
     full_main = real_path(full_dir, DEF_MAIN_TMPL)
 
     if not os.path.exists(full_main):
-        logging.warning(T("Cannot find web template: %s, trying standard template"), full_main)
+        logging.warning_helpful(T("Cannot find web template: %s, trying standard template"), full_main)
         full_dir = real_path(sabnzbd.DIR_INTERFACES, DEF_STDINTF)
         full_main = real_path(full_dir, DEF_MAIN_TMPL)
         if not os.path.exists(full_main):
@@ -320,7 +352,6 @@ def get_user_profile_paths(vista_plus):
     if sabnzbd.DAEMON:
         # In daemon mode, do not try to access the user profile
         # just assume that everything defaults to the program dir
-        sabnzbd.DIR_APPDATA = sabnzbd.DIR_PROG
         sabnzbd.DIR_LCLDATA = sabnzbd.DIR_PROG
         sabnzbd.DIR_HOME = sabnzbd.DIR_PROG
         if sabnzbd.WIN32:
@@ -334,8 +365,6 @@ def get_user_profile_paths(vista_plus):
         try:
             from win32com.shell import shell, shellcon
 
-            path = shell.SHGetFolderPath(0, shellcon.CSIDL_APPDATA, None, 0)
-            sabnzbd.DIR_APPDATA = os.path.join(path, DEF_WORKDIR)
             path = shell.SHGetFolderPath(0, shellcon.CSIDL_LOCAL_APPDATA, None, 0)
             sabnzbd.DIR_LCLDATA = os.path.join(path, DEF_WORKDIR)
             sabnzbd.DIR_HOME = os.environ["USERPROFILE"]
@@ -344,18 +373,16 @@ def get_user_profile_paths(vista_plus):
                 if vista_plus:
                     root = os.environ["AppData"]
                     user = os.environ["USERPROFILE"]
-                    sabnzbd.DIR_APPDATA = "%s\\%s" % (root.replace("\\Roaming", "\\Local"), DEF_WORKDIR)
+                    sabnzbd.DIR_LCLDATA = "%s\\%s" % (root.replace("\\Roaming", "\\Local"), DEF_WORKDIR)
                     sabnzbd.DIR_HOME = user
                 else:
                     root = os.environ["USERPROFILE"]
-                    sabnzbd.DIR_APPDATA = "%s\\%s" % (root, DEF_WORKDIR)
+                    sabnzbd.DIR_LCLDATA = "%s\\%s" % (root, DEF_WORKDIR)
                     sabnzbd.DIR_HOME = root
-                sabnzbd.DIR_LCLDATA = sabnzbd.DIR_APPDATA
             except:
                 pass
 
         # Long-path everything
-        sabnzbd.DIR_APPDATA = long_path(sabnzbd.DIR_APPDATA)
         sabnzbd.DIR_LCLDATA = long_path(sabnzbd.DIR_LCLDATA)
         sabnzbd.DIR_HOME = long_path(sabnzbd.DIR_HOME)
         return
@@ -363,16 +390,14 @@ def get_user_profile_paths(vista_plus):
     elif sabnzbd.DARWIN:
         home = os.environ.get("HOME")
         if home:
-            sabnzbd.DIR_APPDATA = "%s/Library/Application Support/SABnzbd" % home
-            sabnzbd.DIR_LCLDATA = sabnzbd.DIR_APPDATA
+            sabnzbd.DIR_LCLDATA = "%s/Library/Application Support/SABnzbd" % home
             sabnzbd.DIR_HOME = home
             return
     else:
         # Unix/Linux
         home = os.environ.get("HOME")
         if home:
-            sabnzbd.DIR_APPDATA = "%s/.%s" % (home, DEF_WORKDIR)
-            sabnzbd.DIR_LCLDATA = sabnzbd.DIR_APPDATA
+            sabnzbd.DIR_LCLDATA = "%s/.%s" % (home, DEF_WORKDIR)
             sabnzbd.DIR_HOME = home
             return
 
@@ -419,10 +444,12 @@ def print_modules():
         logging.info("UNRAR binary... found (%s)", sabnzbd.newsunpack.RAR_COMMAND)
 
         # Report problematic unrar
-        if sabnzbd.newsunpack.RAR_PROBLEM and not sabnzbd.cfg.ignore_wrong_unrar():
+        if sabnzbd.newsunpack.RAR_PROBLEM:
             have_str = "%.2f" % (float(sabnzbd.newsunpack.RAR_VERSION) / 100)
             want_str = "%.2f" % (float(sabnzbd.constants.REC_RAR_VERSION) / 100)
-            logging.warning(T("Your UNRAR version is %s, we recommend version %s or higher.<br />"), have_str, want_str)
+            logging.warning_helpful(
+                T("Your UNRAR version is %s, we recommend version %s or higher.<br />"), have_str, want_str
+            )
         elif not (sabnzbd.WIN32 or sabnzbd.DARWIN):
             logging.info("UNRAR binary version %.2f", (float(sabnzbd.newsunpack.RAR_VERSION) / 100))
     else:
@@ -495,8 +522,8 @@ def check_resolve(host):
 
 
 def get_webhost(cherryhost, cherryport, https_port):
-    """ Determine the webhost address and port,
-        return (host, port, browserhost)
+    """Determine the webhost address and port,
+    return (host, port, browserhost)
     """
     if cherryhost == "0.0.0.0" and not check_resolve("127.0.0.1"):
         cherryhost = ""
@@ -520,7 +547,7 @@ def get_webhost(cherryhost, cherryport, https_port):
             # Valid user defined name?
             info = socket.getaddrinfo(cherryhost, None)
         except socket.error:
-            if cherryhost not in ("localhost", "127.0.0.1", "::1"):
+            if not is_localhost(cherryhost):
                 cherryhost = "0.0.0.0"
             try:
                 info = socket.getaddrinfo(localhost, None)
@@ -587,12 +614,12 @@ def get_webhost(cherryhost, cherryport, https_port):
             except socket.error:
                 cherryhost = cherryhost.strip("[]")
 
-    if ipv6 and ipv4 and (browserhost not in ("localhost", "127.0.0.1", "[::1]", "::1")):
+    if ipv6 and ipv4 and not is_localhost(browserhost):
         sabnzbd.AMBI_LOCALHOST = True
         logging.info("IPV6 has priority on this system, potential Firefox issue")
 
     if ipv6 and ipv4 and cherryhost == "" and sabnzbd.WIN32:
-        logging.warning(T("Please be aware the 0.0.0.0 hostname will need an IPv6 address for external access"))
+        logging.warning_helpful(T("Please be aware the 0.0.0.0 hostname will need an IPv6 address for external access"))
 
     if cherryhost == "localhost" and not sabnzbd.WIN32 and not sabnzbd.DARWIN:
         # On the Ubuntu family, localhost leads to problems for CherryPy
@@ -602,7 +629,7 @@ def get_webhost(cherryhost, cherryport, https_port):
             if ips[0] != "127.0.0.1":
                 browserhost = "127.0.0.1"
 
-    # This is to please Chrome on OSX
+    # This is to please Chrome on macOS
     if cherryhost == "localhost" and sabnzbd.DARWIN:
         cherryhost = "127.0.0.1"
         browserhost = "localhost"
@@ -667,8 +694,8 @@ def find_free_port(host, currentport):
 
 
 def check_for_sabnzbd(url, upload_nzbs, allow_browser=True):
-    """ Check for a running instance of sabnzbd on this port
-        allow_browser==True|None will launch the browser, False will not.
+    """Check for a running instance of sabnzbd on this port
+    allow_browser==True|None will launch the browser, False will not.
     """
     if allow_browser is None:
         allow_browser = True
@@ -690,10 +717,10 @@ def check_for_sabnzbd(url, upload_nzbs, allow_browser=True):
 
 
 def evaluate_inipath(path):
-    """ Derive INI file path from a partial path.
-        Full file path: if file does not exist the name must contain a dot
-        but not a leading dot.
-        foldername is enough, the standard name will be appended.
+    """Derive INI file path from a partial path.
+    Full file path: if file does not exist the name must contain a dot
+    but not a leading dot.
+    foldername is enough, the standard name will be appended.
     """
     path = os.path.normpath(os.path.abspath(path))
     inipath = os.path.join(path, DEF_INI_FILE)
@@ -710,16 +737,16 @@ def evaluate_inipath(path):
 
 
 def commandline_handler():
-    """ Split win32-service commands are true parameters
-        Returns:
-            service, sab_opts, serv_opts, upload_nzbs
+    """Split win32-service commands are true parameters
+    Returns:
+        service, sab_opts, serv_opts, upload_nzbs
     """
     service = ""
     sab_opts = []
     serv_opts = [os.path.normpath(os.path.abspath(sys.argv[0]))]
     upload_nzbs = []
 
-    # OSX binary: get rid of the weird -psn_0_123456 parameter
+    # macOS binary: get rid of the weird -psn_0_123456 parameter
     for arg in sys.argv:
         if arg.startswith("-psn_"):
             sys.argv.remove(arg)
@@ -833,6 +860,7 @@ def main():
     cherrypylogging = None
     clean_up = False
     logging_level = None
+    console_logging = False
     no_file_log = False
     web_dir = None
     vista_plus = False
@@ -843,7 +871,6 @@ def main():
     pid_path = None
     pid_file = None
     new_instance = False
-    osx_console = False
     ipv6_hosting = None
 
     _service, sab_opts, _serv_opts, upload_nzbs = commandline_handler()
@@ -889,6 +916,8 @@ def main():
             if logging_level < -1 or logging_level > 2:
                 print_help()
                 exit_sab(1)
+        elif opt == "--console":
+            console_logging = True
         elif opt in ("-v", "--version"):
             print_version()
             exit_sab(0)
@@ -930,8 +959,8 @@ def main():
     sabnzbd.DIR_LANGUAGE = real_path(sabnzbd.DIR_PROG, DEF_LANGUAGE)
     org_dir = os.getcwd()
 
-    # Need console logging for SABnzbd.py and SABnzbd-console.exe
-    console_logging = (not hasattr(sys, "frozen")) or (sabnzbd.MY_NAME.lower().find("-console") > 0)
+    # Need console logging if requested, for SABnzbd.py and SABnzbd-console.exe
+    console_logging = console_logging or sabnzbd.MY_NAME.lower().find("-console") > 0 or not hasattr(sys, "frozen")
     console_logging = console_logging and not sabnzbd.DAEMON
 
     LOGLEVELS = (logging.FATAL, logging.WARNING, logging.INFO, logging.DEBUG)
@@ -1106,7 +1135,7 @@ def main():
     try:
         if not no_file_log:
             rollover_log = logging.handlers.RotatingFileHandler(
-                sabnzbd.LOGFILE, "a+", sabnzbd.cfg.log_size.get_int(), sabnzbd.cfg.log_backups()
+                sabnzbd.LOGFILE, "a+", sabnzbd.cfg.log_size(), sabnzbd.cfg.log_backups()
             )
             rollover_log.setFormatter(logging.Formatter(logformat))
             logger.addHandler(rollover_log)
@@ -1128,8 +1157,19 @@ def main():
         if no_file_log:
             logging.info("Console logging only")
 
+    # Start SABnzbd
     logging.info("--------------------------------")
-    logging.info("%s-%s (rev=%s)", sabnzbd.MY_NAME, sabnzbd.__version__, sabnzbd.__baseline__)
+    logging.info("%s-%s", sabnzbd.MY_NAME, sabnzbd.__version__)
+
+    # See if we can get version from git when running an unknown revision
+    if sabnzbd.__baseline__ == "unknown":
+        try:
+            sabnzbd.__baseline__ = sabnzbd.misc.run_command(
+                ["git", "rev-parse", "--short", "HEAD"], cwd=sabnzbd.DIR_PROG
+            ).strip()
+        except:
+            pass
+    logging.info("Commit: %s", sabnzbd.__baseline__)
     logging.info("Full executable path = %s", sabnzbd.MY_FULLNAME)
     if sabnzbd.WIN32:
         suffix = ""
@@ -1153,7 +1193,7 @@ def main():
 
     # On Linux/FreeBSD/Unix "UTF-8" is strongly, strongly adviced:
     if not sabnzbd.WIN32 and not sabnzbd.DARWIN and not ("utf-8" in sabnzbd.encoding.CODEPAGE.lower()):
-        logging.warning(
+        logging.warning_helpful(
             T(
                 "SABnzbd was started with encoding %s, this should be UTF-8. Expect problems with Unicoded file and directory names in downloads."
             ),
@@ -1183,9 +1223,6 @@ def main():
         ctx = ssl.create_default_context()
         logging.debug("Available certificates: %s", repr(ctx.cert_store_stats()))
 
-        # Show IPv4/IPv6 address
-        from sabnzbd.getipaddress import localipv4, publicipv4, ipv6
-
         mylocalipv4 = localipv4()
         if mylocalipv4:
             logging.debug("My local IPv4 address = %s", mylocalipv4)
@@ -1221,7 +1258,7 @@ def main():
     if autobrowser is not None:
         sabnzbd.cfg.autobrowser.set(autobrowser)
 
-    sabnzbd.initialize(pause, clean_up, evalSched=True, repair=repair)
+    sabnzbd.initialize(pause, clean_up, repair=repair)
 
     os.chdir(sabnzbd.DIR_PROG)
 
@@ -1233,7 +1270,7 @@ def main():
     sabnzbd.cfg.web_color.set(sabnzbd.WEB_COLOR)
 
     # Handle the several tray icons
-    if sabnzbd.cfg.win_menu() and not sabnzbd.DAEMON:
+    if sabnzbd.cfg.win_menu() and not sabnzbd.DAEMON and not sabnzbd.WIN_SERVICE:
         if sabnzbd.WIN32:
             import sabnzbd.sabtray
 
@@ -1397,6 +1434,7 @@ def main():
 
     # Make available from both URLs
     main_page = sabnzbd.interface.MainPage()
+    cherrypy.Application.relative_urls = "server"
     cherrypy.tree.mount(main_page, "/", config=appconfig)
     cherrypy.tree.mount(main_page, sabnzbd.cfg.url_base(), config=appconfig)
 
@@ -1465,26 +1503,45 @@ def main():
         check_latest_version()
     autorestarted = False
 
-    # ZeroConfig/Bonjour needs a ip. Lets try to find it.
-    try:
-        z_host = socket.gethostbyname(socket.gethostname())
-    except socket.gaierror:
-        z_host = cherryhost
-    sabnzbd.zconfig.set_bonjour(z_host, cherryport)
+    # Start SSDP and Bonjour if SABnzbd isn't listening on localhost only
+    if sabnzbd.cfg.enable_broadcast() and not is_localhost(cherryhost):
+        # Try to find a LAN IP address for SSDP/Bonjour
+        if is_lan_addr(cherryhost):
+            # A specific listening address was configured, use that
+            external_host = cherryhost
+        else:
+            # Fall back to the IPv4 address of the LAN interface
+            external_host = localipv4()
+        logging.debug("Using %s as host address for Bonjour and SSDP", external_host)
+
+        if is_lan_addr(external_host):
+            sabnzbd.zconfig.set_bonjour(external_host, cherryport)
+
+            # Set URL for browser for external hosts
+            ssdp_url = "%s://%s:%s%s" % (
+                ("https" if enable_https else "http"),
+                external_host,
+                cherryport,
+                sabnzbd.cfg.url_base(),
+            )
+            ssdp.start_ssdp(
+                external_host,
+                "SABnzbd",
+                ssdp_url,
+                "SABnzbd %s" % sabnzbd.__version__,
+                "SABnzbd Team",
+                "https://sabnzbd.org/",
+                "SABnzbd %s" % sabnzbd.__version__,
+                ssdp_broadcast_interval=sabnzbd.cfg.ssdp_broadcast_interval(),
+            )
 
     # Have to keep this running, otherwise logging will terminate
     timer = 0
     while not sabnzbd.SABSTOP:
-        if sabnzbd.LAST_WARNING:
-            msg = sabnzbd.LAST_WARNING
-            sabnzbd.LAST_WARNING = None
-            sabnzbd.notifier.send_notification(T("Warning"), msg, "warning")
-        if sabnzbd.LAST_ERROR:
-            msg = sabnzbd.LAST_ERROR
-            sabnzbd.LAST_ERROR = None
-            sabnzbd.notifier.send_notification(T("Error"), msg, "error")
-
-        time.sleep(3)
+        # Wait to be awoken or every 3 seconds
+        with sabnzbd.SABSTOP_CONDITION:
+            sabnzbd.SABSTOP_CONDITION.wait(3)
+        timer += 1
 
         # Check for loglevel changes
         if LOG_FLAG:
@@ -1494,60 +1551,61 @@ def main():
             if console_logging:
                 console.setLevel(level)
 
-        # 30 sec polling tasks
-        if timer > 9:
+        # 300 sec polling tasks
+        if not timer % 100:
+            if sabnzbd.LOG_ALL:
+                logging.debug("Triggering Python garbage collection")
+            gc.collect()
             timer = 0
+
+        # 30 sec polling tasks
+        if not timer % 10:
             # Keep OS awake (if needed)
             sabnzbd.keep_awake()
             # Restart scheduler (if needed)
-            scheduler.restart()
+            sabnzbd.Scheduler.restart(plan_restart=False)
             # Save config (if needed)
             config.save_config()
             # Check the threads
             if not sabnzbd.check_all_tasks():
                 autorestarted = True
                 sabnzbd.TRIGGER_RESTART = True
-        else:
-            timer += 1
 
         # 3 sec polling tasks
         # Check for auto-restart request
         # Or special restart cases like Mac and WindowsService
         if sabnzbd.TRIGGER_RESTART:
-            # Shutdown
+            logging.info("Performing triggered restart")
             sabnzbd.shutdown_program()
 
-            if sabnzbd.downloader.Downloader.do.paused:
+            # Add arguments and make sure we are in the right directory
+            if sabnzbd.Downloader.paused:
                 sabnzbd.RESTART_ARGS.append("-p")
             if autorestarted:
                 sabnzbd.RESTART_ARGS.append("--autorestarted")
             sys.argv = sabnzbd.RESTART_ARGS
-
             os.chdir(org_dir)
-            # If OSX frozen restart of app instead of embedded python
-            if hasattr(sys, "frozen") and sabnzbd.DARWIN:
-                # [[NSProcessInfo processInfo] processIdentifier]]
-                # logging.info("%s" % (NSProcessInfo.processInfo().processIdentifier()))
-                my_pid = os.getpid()
-                my_name = sabnzbd.MY_FULLNAME.replace("/Contents/MacOS/SABnzbd", "")
-                my_args = " ".join(sys.argv[1:])
-                cmd = 'kill -9 %s && open "%s" --args %s' % (my_pid, my_name, my_args)
-                logging.info("Launching: ", cmd)
-                os.system(cmd)
-            elif sabnzbd.WIN_SERVICE:
-                # Use external service handler to do the restart
-                # Wait 5 seconds to clean up
-                subprocess.Popen("timeout 5 & sc start SABnzbd", shell=True)
+
+            # Binaries require special restart
+            if hasattr(sys, "frozen"):
+                if sabnzbd.DARWIN:
+                    # On macOS restart of app instead of embedded python
+                    my_name = sabnzbd.MY_FULLNAME.replace("/Contents/MacOS/SABnzbd", "")
+                    my_args = " ".join(sys.argv[1:])
+                    cmd = 'kill -9 %s && open "%s" --args %s' % (os.getpid(), my_name, my_args)
+                    logging.info("Launching: %s", cmd)
+                    os.system(cmd)
+                elif sabnzbd.WIN_SERVICE:
+                    # Use external service handler to do the restart
+                    # Wait 5 seconds to clean up
+                    subprocess.Popen("timeout 5 & sc start SABnzbd", shell=True)
+                elif sabnzbd.WIN32:
+                    # Just a simple restart of the exe
+                    os.execv(sys.executable, ['"%s"' % arg for arg in sys.argv])
             else:
+                # CherryPy has special logic to include interpreter options such as "-OO"
                 cherrypy.engine._do_execv()
 
-    config.save_config()
-
-    if sabnzbd.WINTRAY:
-        sabnzbd.WINTRAY.terminate = True
-    if sabnzbd.WIN32:
-        del_connection_info()
-
     # Send our final goodbyes!
     notifier.send_notification("SABnzbd", T("SABnzbd shutdown finished"), "startup")
     logging.info("Leaving SABnzbd")
@@ -1637,13 +1695,14 @@ https://sabnzbd.org/wiki/advanced/sabnzbd-as-a-windows-service
 
 
 def handle_windows_service():
-    """ Handle everything for Windows Service
-        Returns True when any service commands were detected or
-        when we have started as a service.
+    """Handle everything for Windows Service
+    Returns True when any service commands were detected or
+    when we have started as a service.
     """
     # Detect if running as Windows Service (only Vista and above!)
     # Adapted from https://stackoverflow.com/a/55248281/5235502
-    if win32ts.ProcessIdToSessionId(win32api.GetCurrentProcessId()) == 0:
+    # Only works when run from the exe-files
+    if hasattr(sys, "frozen") and win32ts.ProcessIdToSessionId(win32api.GetCurrentProcessId()) == 0:
         servicemanager.Initialize()
         servicemanager.PrepareToHostSingle(SABnzbd)
         servicemanager.StartServiceCtrlDispatcher()
@@ -1691,8 +1750,7 @@ if __name__ == "__main__":
             main()
 
     elif sabnzbd.DARWIN and sabnzbd.FOUNDATION:
-
-        # OSX binary runner
+        # macOS binary runner
         from threading import Thread
         from PyObjCTools import AppHelper
         from AppKit import NSApplication
@@ -1700,7 +1758,7 @@ if __name__ == "__main__":
 
         # Need to run the main application in separate thread because the eventLoop
         # has to be in the main thread. The eventLoop is required for the menu.
-        # This code is made with trial-and-error, please improve!
+        # This code is made with trial-and-error, please feel free to improve!
         class startApp(Thread):
             def run(self):
                 main()
