@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2020 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2021 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -22,25 +22,24 @@ sabnzbd.articlecache - Article cache handling
 import logging
 import threading
 import struct
+from typing import Dict, List
 
 import sabnzbd
 from sabnzbd.decorators import synchronized
 from sabnzbd.constants import GIGI, ANFO, MEBI, LIMIT_DECODE_QUEUE, MIN_DECODE_QUEUE
+from sabnzbd.nzbstuff import Article
 
-# Operations on lists and dicts are atomic, but for
-# the bytes counter we do need a lock
-ARTICLE_LOCK = threading.Lock()
+# Operations on the article table are handled via try/except.
+# The counters need to be made atomic to ensure consistency.
+ARTICLE_COUNTER_LOCK = threading.RLock()
 
 
 class ArticleCache:
-    do = None
-
     def __init__(self):
         self.__cache_limit_org = 0
         self.__cache_limit = 0
         self.__cache_size = 0
-        self.__article_list = []  # List of buffered articles
-        self.__article_table = {}  # Dict of buffered articles
+        self.__article_table: Dict[Article, bytes] = {}  # Dict of buffered articles
 
         # Limit for the decoder is based on the total available cache
         # so it can be larger on memory-rich systems
@@ -52,12 +51,10 @@ class ArticleCache:
         if sabnzbd.DARWIN or sabnzbd.WIN64 or (struct.calcsize("P") * 8) == 64:
             self.__cache_upper_limit = 4 * GIGI
 
-        ArticleCache.do = self
-
     def cache_info(self):
-        return ANFO(len(self.__article_list), abs(self.__cache_size), self.__cache_limit_org)
+        return ANFO(len(self.__article_table), abs(self.__cache_size), self.__cache_limit_org)
 
-    def new_limit(self, limit):
+    def new_limit(self, limit: int):
         """ Called when cache limit changes """
         self.__cache_limit_org = limit
         if limit < 0:
@@ -71,21 +68,21 @@ class ArticleCache:
         # The cache should also not be too small
         self.decoder_cache_article_limit = max(decoder_cache_limit, MIN_DECODE_QUEUE)
 
-    @synchronized(ARTICLE_LOCK)
-    def reserve_space(self, data_size):
+    @synchronized(ARTICLE_COUNTER_LOCK)
+    def reserve_space(self, data_size: int):
         """ Reserve space in the cache """
         self.__cache_size += data_size
 
-    @synchronized(ARTICLE_LOCK)
-    def free_reserved_space(self, data_size):
+    @synchronized(ARTICLE_COUNTER_LOCK)
+    def free_reserved_space(self, data_size: int):
         """ Remove previously reserved space """
         self.__cache_size -= data_size
 
-    def space_left(self):
+    def space_left(self) -> bool:
         """ Is there space left in the set limit? """
         return self.__cache_size < self.__cache_limit
 
-    def save_article(self, article, data):
+    def save_article(self, article: Article, data: bytes):
         """ Save article in cache, either memory or disk """
         nzo = article.nzf.nzo
         if nzo.is_gone():
@@ -108,7 +105,6 @@ class ArticleCache:
             self.reserve_space(data_size)
             if self.space_left():
                 # Add new article to the cache
-                self.__article_list.append(article)
                 self.__article_table[article] = data
             else:
                 # Return the space and save to disk
@@ -118,40 +114,52 @@ class ArticleCache:
             # No data saved in memory, direct to disk
             self.__flush_article_to_disk(article, data)
 
-    def load_article(self, article):
+    def load_article(self, article: Article):
         """ Load the data of the article """
         data = None
         nzo = article.nzf.nzo
 
-        if article in self.__article_list:
-            data = self.__article_table.pop(article)
-            self.__article_list.remove(article)
-            self.free_reserved_space(len(data))
-        elif article.art_id:
-            data = sabnzbd.load_data(article.art_id, nzo.workpath, remove=True, do_pickle=False, silent=True)
-        nzo.remove_saved_article(article)
-
-        return data
-
-    @synchronized(ARTICLE_LOCK)
-    def flush_articles(self):
-        self.__cache_size = 0
-        while self.__article_list:
-            article = self.__article_list.pop(0)
-            data = self.__article_table.pop(article)
-            self.__flush_article_to_disk(article, data)
-
-    def purge_articles(self, articles):
-        """ Remove all saved articles, from memory and disk """
-        for article in articles:
-            if article in self.__article_list:
-                self.__article_list.remove(article)
+        if article in self.__article_table:
+            try:
                 data = self.__article_table.pop(article)
                 self.free_reserved_space(len(data))
-            if article.art_id:
-                sabnzbd.remove_data(article.art_id, article.nzf.nzo.workpath)
+            except KeyError:
+                # Could fail due the article already being deleted by purge_articles, for example
+                # when post-processing deletes the job while delayed articles still come in
+                logging.debug("Failed to load %s from cache, probably already deleted", article)
+                return data
+        elif article.art_id:
+            data = sabnzbd.load_data(article.art_id, nzo.admin_path, remove=True, do_pickle=False, silent=True)
+        nzo.remove_saved_article(article)
+        return data
 
-    def __flush_article_to_disk(self, article, data):
+    def flush_articles(self):
+        logging.debug("Saving %s cached articles to disk", len(self.__article_table))
+        self.__cache_size = 0
+        while self.__article_table:
+            try:
+                article, data = self.__article_table.popitem()
+                self.__flush_article_to_disk(article, data)
+            except KeyError:
+                # Could fail if already deleted by purge_articles or load_data
+                logging.debug("Failed to flush item from cache, probably already deleted or written to disk")
+
+    def purge_articles(self, articles: List[Article]):
+        """ Remove all saved articles, from memory and disk """
+        logging.debug("Purging %s articles from the cache/disk", len(articles))
+        for article in articles:
+            if article in self.__article_table:
+                try:
+                    data = self.__article_table.pop(article)
+                    self.free_reserved_space(len(data))
+                except KeyError:
+                    # Could fail if already deleted by flush_articles or load_data
+                    logging.debug("Failed to flush %s from cache, probably already deleted or written to disk", article)
+            elif article.art_id:
+                sabnzbd.remove_data(article.art_id, article.nzf.nzo.admin_path)
+
+    @staticmethod
+    def __flush_article_to_disk(article: Article, data):
         nzo = article.nzf.nzo
         if nzo.is_gone():
             # Don't store deleted jobs
@@ -159,8 +167,4 @@ class ArticleCache:
 
         # Save data, but don't complain when destination folder is missing
         # because this flush may come after completion of the NZO.
-        sabnzbd.save_data(data, article.get_art_id(), nzo.workpath, do_pickle=False, silent=True)
-
-
-# Create the instance
-ArticleCache()
+        sabnzbd.save_data(data, article.get_art_id(), nzo.admin_path, do_pickle=False, silent=True)
