@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2020 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2021 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -27,6 +27,7 @@ import glob
 
 from Cheetah.Template import Template
 from email.message import EmailMessage
+from email import policy
 
 from sabnzbd.constants import *
 import sabnzbd
@@ -296,4 +297,4 @@ def _prepare_message(txt):
                 msg[keyword] = value
 
     msg.set_content("\n".join(payload))
-    return msg.as_bytes()
+    return msg.as_bytes(policy=msg.policy.clone(linesep="\r\n"))
