@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2020 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2021 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -22,29 +22,29 @@ sabnzbd.encoding - Unicode/byte translation functions
 import locale
 import chardet
 from xml.sax.saxutils import escape
-
+from typing import AnyStr
 
 CODEPAGE = locale.getpreferredencoding()
 
 
-def utob(str_in):
-    """ Shorthand for converting UTF-8 to bytes """
+def utob(str_in: AnyStr) -> bytes:
+    """ Shorthand for converting UTF-8 string to bytes """
     if isinstance(str_in, bytes):
         return str_in
     return str_in.encode("utf-8")
 
 
-def ubtou(str_in):
-    """ Shorthand for converting unicode bytes to UTF-8 """
+def ubtou(str_in: AnyStr) -> str:
+    """ Shorthand for converting unicode bytes to UTF-8 string """
     if not isinstance(str_in, bytes):
         return str_in
     return str_in.decode("utf-8")
 
 
-def platform_btou(str_in):
-    """ Return Unicode, if not already Unicode, decode with locale encoding.
-        NOTE: Used for POpen because universal_newlines/text parameter doesn't
-        always work! We cannot use encoding-parameter because it's Python 3.7+
+def platform_btou(str_in: AnyStr) -> str:
+    """Return Unicode string, if not already Unicode, decode with locale encoding.
+    NOTE: Used for POpen because universal_newlines/text parameter doesn't
+    always work! We cannot use encoding-parameter because it's Python 3.7+
     """
     if isinstance(str_in, bytes):
         try:
@@ -55,11 +55,11 @@ def platform_btou(str_in):
         return str_in
 
 
-def correct_unknown_encoding(str_or_bytes_in):
-    """ Files created on Windows but unpacked/repaired on
-        linux can result in invalid filenames. Try to fix this
-        encoding by going to bytes and then back to unicode again.
-        Last resort we use chardet package
+def correct_unknown_encoding(str_or_bytes_in: AnyStr) -> str:
+    """Files created on Windows but unpacked/repaired on
+    linux can result in invalid filenames. Try to fix this
+    encoding by going to bytes and then back to unicode again.
+    Last resort we use chardet package
     """
     # If already string, back to bytes
     if not isinstance(str_or_bytes_in, bytes):
