@@ -29,15 +29,28 @@ import time
 import fnmatch
 import stat
 import zipfile
+from typing import Union, List, Tuple, Any, Dict, Optional
+
+try:
+    import win32api
+    import win32file
+except ImportError:
+    pass
 
 import sabnzbd
 from sabnzbd.decorators import synchronized
-from sabnzbd.constants import FUTURE_Q_FOLDER, JOB_ADMIN, GIGI
+from sabnzbd.constants import FUTURE_Q_FOLDER, JOB_ADMIN, GIGI, DEF_FILE_MAX
 from sabnzbd.encoding import correct_unknown_encoding
 from sabnzbd.utils import rarfile
 
+# For Windows: determine executable extensions
+if os.name == "nt":
+    PATHEXT = os.environ.get("PATHEXT", "").lower().split(";")
+else:
+    PATHEXT = []
 
-def get_ext(filename):
+
+def get_ext(filename: str) -> str:
     """ Return lowercased file extension """
     try:
         return os.path.splitext(filename)[1].lower()
@@ -45,7 +58,7 @@ def get_ext(filename):
         return ""
 
 
-def get_filename(path):
+def get_filename(path: str) -> str:
     """ Return path without the file extension """
     try:
         return os.path.split(path)[1]
@@ -53,12 +66,12 @@ def get_filename(path):
         return ""
 
 
-def setname_from_path(path):
+def setname_from_path(path: str) -> str:
     """ Get the setname from a path """
     return os.path.splitext(os.path.basename(path))[0]
 
 
-def is_writable(path):
+def is_writable(path: str) -> bool:
     """ Return True is file is writable (also when non-existent) """
     if os.path.isfile(path):
         return bool(os.stat(path).st_mode & stat.S_IWUSR)
@@ -92,10 +105,10 @@ _DEVICES = (
 )
 
 
-def replace_win_devices(name):
-    """ Remove reserved Windows device names from a name.
-        aux.txt ==> _aux.txt
-        txt.aux ==> txt.aux
+def replace_win_devices(name: str) -> str:
+    """Remove reserved Windows device names from a name.
+    aux.txt ==> _aux.txt
+    txt.aux ==> txt.aux
     """
     if name:
         lname = name.lower()
@@ -111,13 +124,13 @@ def replace_win_devices(name):
     return name
 
 
-def has_win_device(p):
-    """ Return True if filename part contains forbidden name
-        Before and after sanitizing
+def has_win_device(filename: str) -> bool:
+    """Return True if filename part contains forbidden name
+    Before and after sanitizing
     """
-    p = os.path.split(p)[1].lower()
+    filename = os.path.split(filename)[1].lower()
     for dev in _DEVICES:
-        if p == dev or p.startswith(dev + ".") or p.startswith("_" + dev + "."):
+        if filename == dev or filename.startswith(dev + ".") or filename.startswith("_" + dev + "."):
             return True
     return False
 
@@ -128,9 +141,9 @@ CH_ILLEGAL_WIN = '\\/<>?*|"\t:'
 CH_LEGAL_WIN = "++{}!@#'+-"
 
 
-def sanitize_filename(name):
-    """ Return filename with illegal chars converted to legal ones
-        and with the par2 extension always in lowercase
+def sanitize_filename(name: str) -> str:
+    """Return filename with illegal chars converted to legal ones
+    and with the par2 extension always in lowercase
     """
     if not name:
         return name
@@ -144,7 +157,7 @@ def sanitize_filename(name):
         legal += CH_LEGAL_WIN
 
     if ":" in name and sabnzbd.DARWIN:
-        # Compensate for the foolish way par2 on OSX handles a colon character
+        # Compensate for the foolish way par2 on macOS handles a colon character
         name = name[name.rfind(":") + 1 :]
 
     lst = []
@@ -160,16 +173,38 @@ def sanitize_filename(name):
     if not name:
         name = "unknown"
 
+    # now split name into name, ext
     name, ext = os.path.splitext(name)
+
+    # If filename is too long (more than DEF_FILE_MAX bytes), brute-force truncate it,
+    # preserving the extension (max ext length 20)
+    # Note: some filesystem can handle up to 255 UTF chars (which is more than 255 bytes) in the filename,
+    # but we stay on the safe side: max DEF_FILE_MAX bytes
+    if len(name.encode("utf8")) + len(ext.encode("utf8")) > DEF_FILE_MAX:
+        logging.debug("Filename %s is too long, so truncating", name + ext)
+        # Too long filenames are often caused by incorrect non-ascii chars,
+        # so brute-force remove those non-ascii chars
+        name = str(name.encode("ascii", "ignore"), "utf-8")
+        # Now it's plain ASCII, so no need for len(str.encode()) anymore; plain len() is enough
+        if len(name) + len(ext) > DEF_FILE_MAX:
+            # still too long, limit the extension
+            maxextlength = 20  # max length of an extension
+            if len(ext) > maxextlength:
+                # allow first <maxextlength> chars, including the starting dot
+                ext = ext[:maxextlength]
+            if len(name) + len(ext) > DEF_FILE_MAX:
+                # Still too long, limit the basename
+                name = name[: DEF_FILE_MAX - len(ext)]
+
     lowext = ext.lower()
     if lowext == ".par2" and lowext != ext:
         ext = lowext
     return name + ext
 
 
-def sanitize_foldername(name):
-    """ Return foldername with dodgy chars converted to safe ones
-        Remove any leading and trailing dot and space characters
+def sanitize_foldername(name: str) -> str:
+    """Return foldername with dodgy chars converted to safe ones
+    Remove any leading and trailing dot and space characters
     """
     if not name:
         return name
@@ -197,6 +232,9 @@ def sanitize_foldername(name):
     if sabnzbd.WIN32 or sabnzbd.cfg.sanitize_safe():
         name = replace_win_devices(name)
 
+    if len(name) >= sabnzbd.cfg.max_foldername_length():
+        name = name[: sabnzbd.cfg.max_foldername_length()]
+
     # And finally, make sure it doesn't end in a dot
     if name != "." and name != "..":
         name = name.rstrip(".")
@@ -206,7 +244,7 @@ def sanitize_foldername(name):
     return name
 
 
-def sanitize_and_trim_path(path):
+def sanitize_and_trim_path(path: str) -> str:
     """ Remove illegal characters and trim element size """
     path = path.strip()
     new_path = ""
@@ -233,8 +271,7 @@ def sanitize_and_trim_path(path):
 
 
 def sanitize_files_in_folder(folder):
-    """ Sanitize each file in the folder, return list of new names
-    """
+    """Sanitize each file in the folder, return list of new names"""
     lst = []
     for root, _, files in os.walk(folder):
         for file_ in files:
@@ -250,17 +287,10 @@ def sanitize_files_in_folder(folder):
     return lst
 
 
-def is_obfuscated_filename(filename):
-    """ Check if this file has an extension, if not, it's
-        probably obfuscated and we don't use it
-    """
-    return len(get_ext(filename)) < 2
-
-
-def real_path(loc, path):
-    """ When 'path' is relative, return normalized join of 'loc' and 'path'
-        When 'path' is absolute, return normalized path
-        A path starting with ~ will be located in the user's Home folder
+def real_path(loc: str, path: str) -> str:
+    """When 'path' is relative, return normalized join of 'loc' and 'path'
+    When 'path' is absolute, return normalized path
+    A path starting with ~ will be located in the user's Home folder
     """
     # The Windows part is a bit convoluted because
     # C: and C:\ are 2 different things
@@ -272,6 +302,9 @@ def real_path(loc, path):
         if not sabnzbd.WIN32 and path.startswith("~/"):
             path = path.replace("~", os.environ.get("HOME", sabnzbd.DIR_HOME), 1)
         if sabnzbd.WIN32:
+            # The Windows-functions work differently on long-path
+            # So we bring it back to normal and make it long-path at the end
+            loc = clip_path(loc)
             path = path.replace("/", "\\")
             if len(path) > 1 and path[0].isalpha() and path[1] == ":":
                 if len(path) == 2 or path[2] != "\\":
@@ -291,13 +324,15 @@ def real_path(loc, path):
     return long_path(os.path.normpath(os.path.abspath(path)))
 
 
-def create_real_path(name, loc, path, umask=False, writable=True):
-    """ When 'path' is relative, create join of 'loc' and 'path'
-        When 'path' is absolute, create normalized path
-        'name' is used for logging.
-        Optional 'umask' will be applied.
-        'writable' means that an existing folder should be writable
-        Returns ('success', 'full path', 'error_msg')
+def create_real_path(
+    name: str, loc: str, path: str, umask: bool = False, writable: bool = True
+) -> Tuple[bool, str, Optional[str]]:
+    """When 'path' is relative, create join of 'loc' and 'path'
+    When 'path' is absolute, create normalized path
+    'name' is used for logging.
+    Optional 'umask' will be applied.
+    'writable' means that an existing folder should be writable
+    Returns ('success', 'full path', 'error_msg')
     """
     if path:
         my_dir = real_path(loc, path)
@@ -319,10 +354,10 @@ def create_real_path(name, loc, path, umask=False, writable=True):
         return False, path, None
 
 
-def same_file(a, b):
-    """ Return 0 if A and B have nothing in common
-        return 1 if A and B are actually the same path
-        return 2 if B is a subfolder of A
+def same_file(a: str, b: str) -> int:
+    """Return 0 if A and B have nothing in common
+    return 1 if A and B are actually the same path
+    return 2 if B is a subfolder of A
     """
     if sabnzbd.WIN32 or sabnzbd.DARWIN:
         a = clip_path(a.lower())
@@ -348,8 +383,8 @@ def same_file(a, b):
             return is_subfolder
 
 
-def is_archive(path):
-    """ Check if file in path is an ZIP, RAR or 7z file
+def is_archive(path: str) -> Tuple[int, Any, str]:
+    """Check if file in path is an ZIP, RAR or 7z file
     :param path: path to file
     :return: (zf, status, expected_extension)
             status: -1==Error/Retry, 0==OK, 1==Ignore
@@ -382,9 +417,9 @@ def is_archive(path):
         return 1, None, ""
 
 
-def check_mount(path):
-    """ Return False if volume isn't mounted on Linux or OSX
-        Retry 6 times with an interval of 1 sec.
+def check_mount(path: str) -> bool:
+    """Return False if volume isn't mounted on Linux or macOS
+    Retry 6 times with an interval of 1 sec.
     """
     if sabnzbd.DARWIN:
         m = re.search(r"^(/Volumes/[^/]+)", path, re.I)
@@ -402,9 +437,9 @@ def check_mount(path):
     return not m
 
 
-def safe_fnmatch(f, pattern):
-    """ fnmatch will fail if the pattern contains any of it's
-        key characters, like [, ] or !.
+def safe_fnmatch(f: str, pattern: str) -> bool:
+    """fnmatch will fail if the pattern contains any of it's
+    key characters, like [, ] or !.
     """
     try:
         return fnmatch.fnmatch(f, pattern)
@@ -412,7 +447,7 @@ def safe_fnmatch(f, pattern):
         return False
 
 
-def globber(path, pattern="*"):
+def globber(path: str, pattern: str = "*") -> List[str]:
     """ Return matching base file/folder names in folder `path` """
     # Cannot use glob.glob() because it doesn't support Windows long name notation
     if os.path.exists(path):
@@ -420,7 +455,7 @@ def globber(path, pattern="*"):
     return []
 
 
-def globber_full(path, pattern="*"):
+def globber_full(path: str, pattern: str = "*") -> List[str]:
     """ Return matching full file/folder names in folder `path` """
     # Cannot use glob.glob() because it doesn't support Windows long name notation
     if os.path.exists(path):
@@ -428,21 +463,10 @@ def globber_full(path, pattern="*"):
     return []
 
 
-def trim_win_path(path):
-    """ Make sure Windows path stays below 70 by trimming last part """
-    if sabnzbd.WIN32 and len(path) > 69:
-        path, folder = os.path.split(path)
-        maxlen = 69 - len(path)
-        if len(folder) > maxlen:
-            folder = folder[:maxlen]
-        path = os.path.join(path, folder).rstrip(". ")
-    return path
-
-
-def fix_unix_encoding(folder):
-    """ Fix bad name encoding for Unix systems
-        This happens for example when files are created
-        on Windows but unpacked/repaired on linux
+def fix_unix_encoding(folder: str):
+    """Fix bad name encoding for Unix systems
+    This happens for example when files are created
+    on Windows but unpacked/repaired on linux
     """
     if not sabnzbd.WIN32 and not sabnzbd.DARWIN:
         for root, dirs, files in os.walk(folder):
@@ -455,12 +479,43 @@ def fix_unix_encoding(folder):
                         logging.info("Cannot correct name of %s", os.path.join(root, name))
 
 
-def make_script_path(script):
+def is_valid_script(basename: str) -> bool:
+    """ Determine if 'basename' is a valid script """
+    return basename in list_scripts(default=False, none=False)
+
+
+def list_scripts(default: bool = False, none: bool = True) -> List[str]:
+    """ Return a list of script names, optionally with 'Default' added """
+    lst = []
+    path = sabnzbd.cfg.script_dir.get_path()
+    if path and os.access(path, os.R_OK):
+        for script in globber_full(path):
+            if os.path.isfile(script):
+                if (
+                    (
+                        sabnzbd.WIN32
+                        and os.path.splitext(script)[1].lower() in PATHEXT
+                        and not win32api.GetFileAttributes(script) & win32file.FILE_ATTRIBUTE_HIDDEN
+                    )
+                    or script.endswith(".py")
+                    or (not sabnzbd.WIN32 and userxbit(script) and not os.path.basename(script).startswith("."))
+                ):
+                    lst.append(os.path.basename(script))
+            # Make sure capitalization is ignored to avoid strange results
+            lst = sorted(lst, key=str.casefold)
+        if none:
+            lst.insert(0, "None")
+        if default:
+            lst.insert(0, "Default")
+    return lst
+
+
+def make_script_path(script: str) -> Optional[str]:
     """ Return full script path, if any valid script exists, else None """
     script_path = None
     script_dir = sabnzbd.cfg.script_dir.get_path()
     if script_dir and script:
-        if script.lower() not in ("none", "default"):
+        if script.lower() not in ("none", "default") and is_valid_script(script):
             script_path = os.path.join(script_dir, script)
             if not os.path.exists(script_path):
                 script_path = None
@@ -470,9 +525,9 @@ def make_script_path(script):
     return script_path
 
 
-def get_admin_path(name, future):
-    """ Return news-style full path to job-admin folder of names job
-        or else the old cache path
+def get_admin_path(name: str, future: bool):
+    """Return news-style full path to job-admin folder of names job
+    or else the old cache path
     """
     if future:
         return os.path.join(sabnzbd.cfg.admin_dir.get_path(), FUTURE_Q_FOLDER)
@@ -480,7 +535,7 @@ def get_admin_path(name, future):
         return os.path.join(os.path.join(sabnzbd.cfg.download_dir.get_path(), name), JOB_ADMIN)
 
 
-def set_chmod(path, permissions, report):
+def set_chmod(path: str, permissions: int, report: bool):
     """ Set 'permissions' on 'path', report any errors when 'report' is True """
     try:
         logging.debug("Applying permissions %s (octal) to %s", oct(permissions), path)
@@ -492,7 +547,7 @@ def set_chmod(path, permissions, report):
             logging.info("Traceback: ", exc_info=True)
 
 
-def set_permissions(path, recursive=True):
+def set_permissions(path: str, recursive: bool = True):
     """ Give folder tree and its files their proper permissions """
     if not sabnzbd.WIN32:
         umask = sabnzbd.cfg.umask()
@@ -523,14 +578,28 @@ def set_permissions(path, recursive=True):
             set_chmod(path, umask_file, report)
 
 
-def clip_path(path):
+def userxbit(filename: str) -> bool:
+    """Returns boolean if the x-bit for user is set on the given file.
+    This is a workaround: os.access(filename, os.X_OK) does not work
+    on certain mounted file systems. Does not work at all on Windows.
+    """
+    # rwx rwx rwx
+    # 876 543 210      # we want bit 6 from the right, counting from 0
+    userxbit = 1 << 6  # bit 6
+    rwxbits = os.stat(filename)[0]  # the first element of os.stat() is "mode"
+    # do logical AND, check if it is not 0:
+    xbitset = (rwxbits & userxbit) > 0
+    return xbitset
+
+
+def clip_path(path: str) -> str:
     r""" Remove \\?\ or \\?\UNC\ prefix from Windows path """
     if sabnzbd.WIN32 and path and "?" in path:
         path = path.replace("\\\\?\\UNC\\", "\\\\", 1).replace("\\\\?\\", "", 1)
     return path
 
 
-def long_path(path):
+def long_path(path: str) -> str:
     """ For Windows, convert to long style path; others, return same path """
     if sabnzbd.WIN32 and path and not path.startswith("\\\\?\\"):
         if path.startswith("\\\\"):
@@ -549,15 +618,18 @@ DIR_LOCK = threading.RLock()
 
 
 @synchronized(DIR_LOCK)
-def create_all_dirs(path, apply_umask=False):
-    """ Create all required path elements and set umask on all
-        The umask argument is ignored on Windows
-        Return path if elements could be made or exists
+def create_all_dirs(path: str, apply_umask: bool = False) -> Union[str, bool]:
+    """Create all required path elements and set umask on all
+    The umask argument is ignored on Windows
+    Return path if elements could be made or exists
     """
     try:
         logging.info("Creating directories: %s", path)
         if sabnzbd.WIN32:
-            os.makedirs(path, exist_ok=True)
+            # On Windows it can fail on UNC-paths in long-path notation
+            # https://bugs.python.org/issue41705
+            if not os.path.exists(path):
+                os.makedirs(path)
         else:
             # We need to build the directory recursively so we can
             # apply permissions to only the newly created folders
@@ -584,7 +656,7 @@ def create_all_dirs(path, apply_umask=False):
 
 
 @synchronized(DIR_LOCK)
-def get_unique_path(dirpath, n=0, create_dir=True):
+def get_unique_path(dirpath: str, n: int = 0, create_dir: bool = True) -> str:
     """ Determine a unique folder or filename """
 
     if not check_mount(dirpath):
@@ -604,9 +676,9 @@ def get_unique_path(dirpath, n=0, create_dir=True):
 
 
 @synchronized(DIR_LOCK)
-def get_unique_filename(path):
-    """ Check if path is unique.
-        If not, add number like: "/path/name.NUM.ext".
+def get_unique_filename(path: str) -> str:
+    """Check if path is unique.
+    If not, add number like: "/path/name.NUM.ext".
     """
     num = 1
     new_path, fname = os.path.split(path)
@@ -619,7 +691,7 @@ def get_unique_filename(path):
 
 
 @synchronized(DIR_LOCK)
-def listdir_full(input_dir, recursive=True):
+def listdir_full(input_dir: str, recursive: bool = True) -> List[str]:
     """ List all files in dirs and sub-dirs """
     filelist = []
     for root, dirs, files in os.walk(input_dir):
@@ -633,9 +705,9 @@ def listdir_full(input_dir, recursive=True):
 
 
 @synchronized(DIR_LOCK)
-def move_to_path(path, new_path):
-    """ Move a file to a new path, optionally give unique filename
-        Return (ok, new_path)
+def move_to_path(path: str, new_path: str) -> Tuple[bool, Optional[str]]:
+    """Move a file to a new path, optionally give unique filename
+    Return (ok, new_path)
     """
     ok = True
     overwrite = sabnzbd.cfg.overwrite_files()
@@ -673,7 +745,7 @@ def move_to_path(path, new_path):
 
 
 @synchronized(DIR_LOCK)
-def cleanup_empty_directories(path):
+def cleanup_empty_directories(path: str):
     """ Remove all empty folders inside (and including) 'path' """
     path = os.path.normpath(path)
     while 1:
@@ -697,7 +769,7 @@ def cleanup_empty_directories(path):
 
 
 @synchronized(DIR_LOCK)
-def get_filepath(path, nzo, filename):
+def get_filepath(path: str, nzo, filename: str):
     """ Create unique filepath """
     # This procedure is only used by the Assembler thread
     # It does no umask setting
@@ -734,8 +806,9 @@ def get_filepath(path, nzo, filename):
 
 
 @synchronized(DIR_LOCK)
-def renamer(old, new):
-    """ Rename file/folder with retries for Win32 """
+def renamer(old: str, new: str, create_local_directories: bool = False):
+    """Rename file/folder with retries for Win32
+    Optionally alows the creation of local directories if they don't exist yet"""
     # Sanitize last part of new name
     path, name = os.path.split(new)
     new = os.path.join(path, sanitize_filename(name))
@@ -744,13 +817,26 @@ def renamer(old, new):
     if old == new:
         return
 
+    # In case we want nonexistent directories to be created, check for directory escape (forbidden)
+    if create_local_directories:
+        oldpath, _ = os.path.split(old)
+        # Check not outside directory
+        # In case of "same_file() == 1": same directory, so nothing to do
+        if same_file(oldpath, path) == 0:
+            # Outside current directory, this is most likely malicious
+            logging.error(T("Blocked attempt to create directory %s"), path)
+            raise OSError("Refusing to go outside directory")
+        elif same_file(oldpath, path) == 2:
+            # Sub-directory, so create if does not yet exist:
+            create_all_dirs(path)
+
     logging.debug('Renaming "%s" to "%s"', old, new)
     if sabnzbd.WIN32:
-        retries = 15
+        retries = 10
         while retries > 0:
             try:
                 # First we try 3 times with os.rename
-                if retries > 12:
+                if retries > 7:
                     os.rename(old, new)
                 else:
                     # Now we try the back-up method
@@ -763,11 +849,12 @@ def renamer(old, new):
                     # Error 17 - Rename can't move to different disk
                     # Jump to moving with shutil.move
                     retries -= 3
-                elif err.winerror == 32:
+                elif err.winerror == 32 or err.winerror == 5:
                     # Error 32 - Used by another process
+                    # Error 5 - Access is denied (virus scanners)
                     logging.debug("File busy, retrying rename %s to %s", old, new)
                     retries -= 1
-                    # Wait for the other process to finish
+                    # Wait for the other process
                     time.sleep(2)
                 else:
                     raise
@@ -776,14 +863,14 @@ def renamer(old, new):
         shutil.move(old, new)
 
 
-def remove_file(path):
+def remove_file(path: str):
     """ Wrapper function so any file removal is logged """
     logging.debug("[%s] Deleting file %s", sabnzbd.misc.caller_name(), path)
     os.remove(path)
 
 
 @synchronized(DIR_LOCK)
-def remove_dir(path):
+def remove_dir(path: str):
     """ Remove directory with retries for Win32 """
     logging.debug("[%s] Removing dir %s", sabnzbd.misc.caller_name(), path)
     if sabnzbd.WIN32:
@@ -806,7 +893,7 @@ def remove_dir(path):
 
 
 @synchronized(DIR_LOCK)
-def remove_all(path, pattern="*", keep_folder=False, recursive=False):
+def remove_all(path: str, pattern: str = "*", keep_folder: bool = False, recursive: bool = False):
     """ Remove folder and all its content (optionally recursive) """
     if path and os.path.exists(path):
         # Fast-remove the whole tree if recursive
@@ -840,67 +927,47 @@ def remove_all(path, pattern="*", keep_folder=False, recursive=False):
 ##############################################################################
 # Diskfree
 ##############################################################################
-def find_dir(p):
-    """ Return first folder level that exists in this path """
+def diskspace_base(dir_to_check: str) -> Tuple[float, float]:
+    """ Return amount of free and used diskspace in GBytes """
+    # Find first folder level that exists in the path
     x = "x"
-    while x and not os.path.exists(p):
-        p, x = os.path.split(p)
-    return p
+    while x and not os.path.exists(dir_to_check):
+        dir_to_check, x = os.path.split(dir_to_check)
 
-
-if sabnzbd.WIN32:
-    # windows diskfree
-    try:
-        # Careful here, because win32api test hasn't been done yet!
-        import win32api
-    except:
-        pass
-
-    def diskspace_base(_dir):
-        """ Return amount of free and used diskspace in GBytes """
-        _dir = find_dir(_dir)
+    if sabnzbd.WIN32:
+        # windows diskfree
         try:
-            available, disk_size, total_free = win32api.GetDiskFreeSpaceEx(_dir)
+            available, disk_size, total_free = win32api.GetDiskFreeSpaceEx(dir_to_check)
             return disk_size / GIGI, available / GIGI
         except:
             return 0.0, 0.0
-
-
-else:
-    try:
-        os.statvfs
+    elif hasattr(os, "statvfs"):
         # posix diskfree
-        def diskspace_base(_dir):
-            """ Return amount of free and used diskspace in GBytes """
-            _dir = find_dir(_dir)
-            try:
-                s = os.statvfs(_dir)
-                if s.f_blocks < 0:
-                    disk_size = float(sys.maxsize) * float(s.f_frsize)
-                else:
-                    disk_size = float(s.f_blocks) * float(s.f_frsize)
-                if s.f_bavail < 0:
-                    available = float(sys.maxsize) * float(s.f_frsize)
-                else:
-                    available = float(s.f_bavail) * float(s.f_frsize)
-                return disk_size / GIGI, available / GIGI
-            except:
-                return 0.0, 0.0
-
-    except ImportError:
-
-        def diskspace_base(_dir):
-            return 20.0, 10.0
+        try:
+            s = os.statvfs(dir_to_check)
+            if s.f_blocks < 0:
+                disk_size = float(sys.maxsize) * float(s.f_frsize)
+            else:
+                disk_size = float(s.f_blocks) * float(s.f_frsize)
+            if s.f_bavail < 0:
+                available = float(sys.maxsize) * float(s.f_frsize)
+            else:
+                available = float(s.f_bavail) * float(s.f_frsize)
+            return disk_size / GIGI, available / GIGI
+        except:
+            return 0.0, 0.0
+    else:
+        return 20.0, 10.0
 
 
 # Store all results to speed things up
 __DIRS_CHECKED = []
 __DISKS_SAME = None
-__LAST_DISK_RESULT = {"download_dir": [], "complete_dir": []}
+__LAST_DISK_RESULT = {"download_dir": (0.0, 0.0), "complete_dir": (0.0, 0.0)}
 __LAST_DISK_CALL = 0
 
 
-def diskspace(force=False):
+def diskspace(force: bool = False) -> Dict[str, Tuple[float, float]]:
     """ Wrapper to cache results """
     global __DIRS_CHECKED, __DISKS_SAME, __LAST_DISK_RESULT, __LAST_DISK_CALL
 
