@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2020 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2021 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -30,22 +30,43 @@ import time
 import datetime
 import inspect
 import ctypes
+import ipaddress
+from typing import Union, Tuple, Any, AnyStr, Optional, List
 
 import sabnzbd
 from sabnzbd.constants import DEFAULT_PRIORITY, MEBI, DEF_ARTICLE_CACHE_DEFAULT, DEF_ARTICLE_CACHE_MAX
 import sabnzbd.config as config
 import sabnzbd.cfg as cfg
 from sabnzbd.encoding import ubtou, platform_btou
+from sabnzbd.filesystem import userxbit
 
 TAB_UNITS = ("", "K", "M", "G", "T", "P")
-RE_UNITS = re.compile(r"(\d+\.*\d*)\s*([KMGTP]{0,1})", re.I)
+RE_UNITS = re.compile(r"(\d+\.*\d*)\s*([KMGTP]?)", re.I)
 RE_VERSION = re.compile(r"(\d+)\.(\d+)\.(\d+)([a-zA-Z]*)(\d*)")
-RE_IP4 = re.compile(r"inet\s+(addr:\s*){0,1}(\d+\.\d+\.\d+\.\d+)")
-RE_IP6 = re.compile(r"inet6\s+(addr:\s*){0,1}([0-9a-f:]+)", re.I)
+RE_IP4 = re.compile(r"inet\s+(addr:\s*)?(\d+\.\d+\.\d+\.\d+)")
+RE_IP6 = re.compile(r"inet6\s+(addr:\s*)?([0-9a-f:]+)", re.I)
 
 # Check if strings are defined for AM and PM
 HAVE_AMPM = bool(time.strftime("%p", time.localtime()))
 
+if sabnzbd.WIN32:
+    try:
+        import win32process
+        import win32con
+
+        # Define scheduling priorities
+        WIN_SCHED_PRIOS = {
+            1: win32process.IDLE_PRIORITY_CLASS,
+            2: win32process.BELOW_NORMAL_PRIORITY_CLASS,
+            3: win32process.NORMAL_PRIORITY_CLASS,
+            4: win32process.ABOVE_NORMAL_PRIORITY_CLASS,
+        }
+    except ImportError:
+        pass
+
+if sabnzbd.DARWIN:
+    from PyObjCTools import AppHelper
+
 
 def time_format(fmt):
     """ Return time-format string adjusted for 12/24 hour clock setting """
@@ -55,10 +76,10 @@ def time_format(fmt):
         return fmt
 
 
-def calc_age(date, trans=False):
-    """ Calculate the age difference between now and date.
-        Value is returned as either days, hours, or minutes.
-        When 'trans' is True, time symbols will be translated.
+def calc_age(date: datetime.datetime, trans=False) -> str:
+    """Calculate the age difference between now and date.
+    Value is returned as either days, hours, or minutes.
+    When 'trans' is True, time symbols will be translated.
     """
     if trans:
         d = T("d")  # : Single letter abbreviation of day
@@ -89,16 +110,7 @@ def calc_age(date, trans=False):
     return age
 
 
-def monthrange(start, finish):
-    """ Calculate months between 2 dates, used in the Config template """
-    months = (finish.year - start.year) * 12 + finish.month + 1
-    for i in range(start.month, months):
-        year = (i - 1) / 12 + start.year
-        month = (i - 1) % 12 + 1
-        yield datetime.date(int(year), int(month), 1)
-
-
-def safe_lower(txt):
+def safe_lower(txt: Any) -> str:
     """ Return lowercased string. Return '' for None """
     if txt:
         return txt.lower()
@@ -130,20 +142,19 @@ def name_to_cat(fname, cat=None):
     return fname, cat
 
 
-def cat_to_opts(cat, pp=None, script=None, priority=None):
-    """ Derive options from category, if options not already defined.
-        Specified options have priority over category-options.
-        If no valid category is given, special category '*' will supply default values
+def cat_to_opts(cat, pp=None, script=None, priority=None) -> Tuple[str, int, str, int]:
+    """Derive options from category, if options not already defined.
+    Specified options have priority over category-options.
+    If no valid category is given, special category '*' will supply default values
     """
-    def_cat = config.get_categories("*")
+    def_cat = config.get_category()
     cat = safe_lower(cat)
     if cat in ("", "none", "default"):
         cat = "*"
-    try:
-        my_cat = config.get_categories()[cat]
-    except KeyError:
+    my_cat = config.get_category(cat)
+    # Ignore the input category if we don't know it
+    if my_cat == def_cat:
         cat = "*"
-        my_cat = def_cat
 
     if pp is None:
         pp = my_cat.pp()
@@ -160,11 +171,11 @@ def cat_to_opts(cat, pp=None, script=None, priority=None):
         if priority == DEFAULT_PRIORITY:
             priority = def_cat.priority()
 
-    logging.debug("Cat->Attrib cat=%s pp=%s script=%s prio=%s", cat, pp, script, priority)
+    logging.debug("Parsing category %s to attributes: pp=%s script=%s prio=%s", cat, pp, script, priority)
     return cat, pp, script, priority
 
 
-def pp_to_opts(pp):
+def pp_to_opts(pp: int) -> Tuple[bool, bool, bool]:
     """ Convert numeric processing options to (repair, unpack, delete) """
     # Convert the pp to an int
     pp = sabnzbd.interface.int_conv(pp)
@@ -177,10 +188,8 @@ def pp_to_opts(pp):
     return True, True, True
 
 
-def opts_to_pp(repair, unpack, delete):
+def opts_to_pp(repair: bool, unpack: bool, delete: bool) -> int:
     """ Convert (repair, unpack, delete) to numeric process options """
-    if repair is None:
-        return None
     pp = 0
     if repair:
         pp = 1
@@ -215,10 +224,10 @@ def wildcard_to_re(text):
 
 
 def cat_convert(cat):
-    """ Convert indexer's category/group-name to user categories.
-        If no match found, but indexer-cat equals user-cat, then return user-cat
-        If no match found, but the indexer-cat starts with the user-cat, return user-cat
-        If no match found, return None
+    """Convert indexer's category/group-name to user categories.
+    If no match found, but indexer-cat equals user-cat, then return user-cat
+    If no match found, but the indexer-cat starts with the user-cat, return user-cat
+    If no match found, return None
     """
     if cat and cat.lower() != "none":
         cats = config.get_ordered_categories()
@@ -255,8 +264,8 @@ def cat_convert(cat):
 
 
 def windows_variant():
-    """ Determine Windows variant
-        Return vista_plus, x64
+    """Determine Windows variant
+    Return vista_plus, x64
     """
     from win32api import GetVersionEx
     from win32con import VER_PLATFORM_WIN32_NT
@@ -329,7 +338,7 @@ def set_serv_parms(service, args):
     return True
 
 
-def get_from_url(url):
+def get_from_url(url: str) -> Optional[str]:
     """ Retrieve URL and return content """
     try:
         req = urllib.request.Request(url)
@@ -360,26 +369,26 @@ def convert_version(text):
 
 
 def check_latest_version():
-    """ Do an online check for the latest version
+    """Do an online check for the latest version
 
-        Perform an online version check
-        Syntax of online version file:
-            <current-final-release>
-            <url-of-current-final-release>
-            <latest-alpha/beta-or-rc>
-            <url-of-latest-alpha/beta/rc-release>
-        The latter two lines are only present when an alpha/beta/rc is available.
-        Formula for the version numbers (line 1 and 3).
-            <major>.<minor>.<bugfix>[rc|beta|alpha]<cand>
+    Perform an online version check
+    Syntax of online version file:
+        <current-final-release>
+        <url-of-current-final-release>
+        <latest-alpha/beta-or-rc>
+        <url-of-latest-alpha/beta/rc-release>
+    The latter two lines are only present when an alpha/beta/rc is available.
+    Formula for the version numbers (line 1 and 3).
+        <major>.<minor>.<bugfix>[rc|beta|alpha]<cand>
 
-        The <cand> value for a final version is assumned to be 99.
-        The <cand> value for the beta/rc version is 1..98, with RC getting
-        a boost of 80 and Beta of 40.
-        This is done to signal alpha/beta/rc users of availability of the final
-        version (which is implicitly 99).
-        People will only be informed to upgrade to a higher alpha/beta/rc version, if
-        they are already using an alpha/beta/rc.
-        RC's are valued higher than Beta's, which are valued higher than Alpha's.
+    The <cand> value for a final version is assumned to be 99.
+    The <cand> value for the beta/rc version is 1..98, with RC getting
+    a boost of 80 and Beta of 40.
+    This is done to signal alpha/beta/rc users of availability of the final
+    version (which is implicitly 99).
+    People will only be informed to upgrade to a higher alpha/beta/rc version, if
+    they are already using an alpha/beta/rc.
+    RC's are valued higher than Beta's, which are valued higher than Alpha's.
     """
 
     if not cfg.version_check():
@@ -430,16 +439,20 @@ def check_latest_version():
         latest_label = latest_testlabel
         url = url_beta
 
-    if testver and current < latest:
-        # This is a test version, but user has't seen the
+    notify_version = None
+    if current < latest:
+        # This is a test version, but user hasn't seen the
         # "Final" of this one yet, so show the Final
+        # Or this one is behind, show latest final
         sabnzbd.NEW_VERSION = (latest_label, url)
-    elif current < latest:
-        # This one is behind, show latest final
-        sabnzbd.NEW_VERSION = (latest_label, url)
+        notify_version = latest_label
     elif testver and current < latest_test:
         # This is a test version beyond the latest Final, so show latest Alpha/Beta/RC
         sabnzbd.NEW_VERSION = (latest_testlabel, url_beta)
+        notify_version = latest_testlabel
+
+    if notify_version:
+        sabnzbd.notifier.send_notification(T("Update Available!"), "SABnzbd %s" % notify_version, "other")
 
 
 def upload_file_to_sabnzbd(url, fp):
@@ -459,15 +472,15 @@ def upload_file_to_sabnzbd(url, fp):
                 url = "%s&ma_username=%s&ma_password=%s" % (url, username, password)
         get_from_url(url)
     except:
-        logging.error("Failed to upload file: %s", fp)
+        logging.error(T("Failed to upload file: %s"), fp)
         logging.info("Traceback: ", exc_info=True)
 
 
-def from_units(val):
+def from_units(val: str) -> float:
     """ Convert K/M/G/T/P notation to float """
     val = str(val).strip().upper()
     if val == "-1":
-        return val
+        return float(val)
     m = RE_UNITS.search(val)
     if m:
         if m.group(2):
@@ -487,9 +500,9 @@ def from_units(val):
         return 0.0
 
 
-def to_units(val, postfix=""):
-    """ Convert number to K/M/G/T/P notation
-        Show single decimal for M and higher
+def to_units(val: Union[int, float], postfix="") -> str:
+    """Convert number to K/M/G/T/P notation
+    Show single decimal for M and higher
     """
     dec_limit = 1
     if val < 0:
@@ -518,8 +531,8 @@ def to_units(val, postfix=""):
 
 def caller_name(skip=2):
     """Get a name of a caller in the format module.method
-       Originally used: https://gist.github.com/techtonik/2151727
-       Adapted for speed by using sys calls directly
+    Originally used: https://gist.github.com/techtonik/2151727
+    Adapted for speed by using sys calls directly
     """
     # Only do the tracing on Debug (function is always called)
     if cfg.log_level() != 2:
@@ -528,7 +541,7 @@ def caller_name(skip=2):
     parentframe = sys._getframe(skip)
     function_name = parentframe.f_code.co_name
 
-    # Modulename not available in the binaries, we can use the filename instead
+    # Module name is not available in the binaries, we can use the filename instead
     if hasattr(sys, "frozen"):
         module_name = inspect.getfile(parentframe)
     else:
@@ -541,16 +554,12 @@ def caller_name(skip=2):
     return ".".join([module_name, function_name])
 
 
-def exit_sab(value):
+def exit_sab(value: int):
     """ Leave the program after flushing stderr/stdout """
     sys.stderr.flush()
     sys.stdout.flush()
-    if hasattr(sys, "frozen") and sabnzbd.DARWIN:
-        sabnzbd.SABSTOP = True
-        from PyObjCTools import AppHelper
-
-        AppHelper.stopEventLoop()
-    sys.exit(value)
+    # Cannot use sys.exit as it will not work inside the macOS-runner-thread
+    os._exit(value)
 
 
 def split_host(srv):
@@ -576,9 +585,9 @@ def split_host(srv):
 
 
 def get_cache_limit():
-    """ Depending on OS, calculate cache limits.
-        In ArticleCache it will make sure we stay
-        within system limits for 32/64 bit
+    """Depending on OS, calculate cache limits.
+    In ArticleCache it will make sure we stay
+    within system limits for 32/64 bit
     """
     # Calculate, if possible
     try:
@@ -641,7 +650,7 @@ def get_windows_memory():
 
 def get_darwin_memory():
     """ Use system-call to extract total memory on macOS """
-    system_output = sabnzbd.newsunpack.run_simple(["sysctl", "hw.memsize"])
+    system_output = run_command(["sysctl", "hw.memsize"])
     return float(system_output.split()[1])
 
 
@@ -733,7 +742,7 @@ def format_time_string(seconds):
     return " ".join(completestr)
 
 
-def int_conv(value):
+def int_conv(value: Any) -> int:
     """ Safe conversion to int (can handle None) """
     try:
         value = int(value)
@@ -772,11 +781,7 @@ def get_all_passwords(nzo):
         meta_passwords.append(pw)
 
     if meta_passwords:
-        if nzo.password == meta_passwords[0]:
-            # this nzo.password came from meta, so don't use it twice
-            passwords.extend(meta_passwords[1:])
-        else:
-            passwords.extend(meta_passwords)
+        passwords.extend(meta_passwords)
         logging.info("Read %s passwords from meta data in NZB: %s", len(meta_passwords), meta_passwords)
 
     pw_file = cfg.password_file.get_path()
@@ -792,13 +797,14 @@ def get_all_passwords(nzo):
 
             # Check size
             if len(pws) > 30:
-                logging.warning(
+                logging.warning_helpful(
                     T(
                         "Your password file contains more than 30 passwords, testing all these passwords takes a lot of time. Try to only list useful passwords."
                     )
                 )
         except:
-            logging.warning("Failed to read the passwords file %s", pw_file)
+            logging.warning(T("Failed to read the password file %s"), pw_file)
+            logging.info("Traceback: ", exc_info=True)
 
     if nzo.password:
         # If an explicit password was set, add a retry without password, just in case.
@@ -807,7 +813,7 @@ def get_all_passwords(nzo):
         # If we're not sure about encryption, start with empty password
         # and make sure we have at least the empty password
         passwords.insert(0, "")
-    return passwords
+    return set(passwords)
 
 
 def find_on_path(targets):
@@ -828,23 +834,50 @@ def find_on_path(targets):
     return None
 
 
-def probablyipv4(ip):
-    if ip.count(".") == 3 and re.sub("[0123456789.]", "", ip) == "":
-        return True
-    else:
+def is_ipv4_addr(ip: str) -> bool:
+    """ Determine if the ip is an IPv4 address """
+    try:
+        return ipaddress.ip_address(ip).version == 4
+    except ValueError:
         return False
 
 
-def probablyipv6(ip):
-    # Returns True if the given input is probably an IPv6 address
-    # Square Brackets like '[2001::1]' are OK
-    if ip.count(":") >= 2 and re.sub(r"[0123456789abcdefABCDEF:\[\]]", "", ip) == "":
-        return True
-    else:
+def is_ipv6_addr(ip: str) -> bool:
+    """ Determine if the ip is an IPv6 address; square brackets ([2001::1]) are OK """
+    try:
+        return ipaddress.ip_address(ip.strip("[]")).version == 6
+    except (ValueError, AttributeError):
         return False
 
 
-def ip_extract():
+def is_loopback_addr(ip: str) -> bool:
+    """ Determine if the ip is an IPv4 or IPv6 local loopback address """
+    try:
+        if ip.find(".") < 0:
+            ip = ip.strip("[]")
+        return ipaddress.ip_address(ip).is_loopback
+    except (ValueError, AttributeError):
+        return False
+
+
+def is_localhost(value: str) -> bool:
+    """ Determine if the input is some variety of 'localhost' """
+    return (value == "localhost") or is_loopback_addr(value)
+
+
+def is_lan_addr(ip: str) -> bool:
+    """ Determine if the ip is a local area network address """
+    try:
+        return (
+            ip not in ("0.0.0.0", "255.255.255.255", "::")
+            and ipaddress.ip_address(ip).is_private
+            and not is_loopback_addr(ip)
+        )
+    except ValueError:
+        return False
+
+
+def ip_extract() -> List[str]:
     """ Return list of IP addresses of this system """
     ips = []
     program = find_on_path("ip")
@@ -864,17 +897,7 @@ def ip_extract():
         for item in info:
             ips.append(item[4][0])
     else:
-        p = subprocess.Popen(
-            program,
-            shell=False,
-            stdin=subprocess.PIPE,
-            stdout=subprocess.PIPE,
-            stderr=subprocess.STDOUT,
-            startupinfo=None,
-            creationflags=0,
-        )
-        output = platform_btou(p.stdout.read())
-        p.wait()
+        output = run_command(program)
         for line in output.split("\n"):
             m = RE_IP4.search(line)
             if not (m and m.group(2)):
@@ -884,9 +907,45 @@ def ip_extract():
     return ips
 
 
-def get_base_url(url):
-    """ Return only the true root domain for the favicon, so api.oznzb.com -> oznzb.com
-        But also api.althub.co.za -> althub.co.za
+def get_server_addrinfo(host: str, port: int) -> socket.getaddrinfo:
+    """ Return processed getaddrinfo() """
+    try:
+        int(port)
+    except:
+        port = 119
+    opt = sabnzbd.cfg.ipv6_servers()
+    """ ... with the following meaning for 'opt':
+    Control the use of IPv6 Usenet server addresses. Meaning:
+    0 = don't use
+    1 = use when available and reachable (DEFAULT)
+    2 = force usage (when SABnzbd's detection fails)
+    """
+    try:
+        # Standard IPV4 or IPV6
+        ips = socket.getaddrinfo(host, port, 0, socket.SOCK_STREAM)
+        if opt == 2 or (opt == 1 and sabnzbd.EXTERNAL_IPV6) or (opt == 1 and sabnzbd.cfg.load_balancing() == 2):
+            # IPv6 forced by user, or IPv6 allowed and reachable, or IPv6 allowed and loadbalancing-with-IPv6 activated
+            # So return all IP addresses, no matter IPv4 or IPv6:
+            return ips
+        else:
+            # IPv6 unreachable or not allowed by user, so only return IPv4 address(es):
+            return [ip for ip in ips if ":" not in ip[4][0]]
+    except:
+        if opt == 2 or (opt == 1 and sabnzbd.EXTERNAL_IPV6) or (opt == 1 and sabnzbd.cfg.load_balancing() == 2):
+            try:
+                # Try IPV6 explicitly
+                return socket.getaddrinfo(
+                    host, port, socket.AF_INET6, socket.SOCK_STREAM, socket.IPPROTO_IP, socket.AI_CANONNAME
+                )
+            except:
+                # Nothing found!
+                pass
+        return []
+
+
+def get_base_url(url: str) -> str:
+    """Return only the true root domain for the favicon, so api.oznzb.com -> oznzb.com
+    But also api.althub.co.za -> althub.co.za
     """
     url_host = urllib.parse.urlparse(url).hostname
     if url_host:
@@ -899,7 +958,7 @@ def get_base_url(url):
         return ""
 
 
-def match_str(text, matches):
+def match_str(text: AnyStr, matches: Tuple[AnyStr, ...]) -> Optional[AnyStr]:
     """ Return first matching element of list 'matches' in 'text', otherwise None """
     for match in matches:
         if match in text:
@@ -907,7 +966,7 @@ def match_str(text, matches):
     return None
 
 
-def nntp_to_msg(text):
+def nntp_to_msg(text: Union[List[AnyStr], str]) -> str:
     """ Format raw NNTP bytes data for display """
     if isinstance(text, list):
         text = text[0]
@@ -919,3 +978,83 @@ def nntp_to_msg(text):
     else:
         lines = text.split(b"\r\n")
         return ubtou(lines[0])
+
+
+def list2cmdline(lst: List[str]) -> str:
+    """ convert list to a cmd.exe-compatible command string """
+    nlst = []
+    for arg in lst:
+        if not arg:
+            nlst.append('""')
+        else:
+            nlst.append('"%s"' % arg)
+    return " ".join(nlst)
+
+
+def build_and_run_command(command: List[str], flatten_command=False, **kwargs):
+    """Builds and then runs command with nessecary flags and optional
+    IONice and Nice commands. Optional Popen arguments can be supplied.
+    On Windows we need to run our own list2cmdline for Unrar.
+    Returns the Popen-instance.
+    """
+    # command[0] should be set, and thus not None
+    if not command[0]:
+        logging.error(T("[%s] The command in build_command is undefined."), caller_name())
+        raise IOError
+
+    if not sabnzbd.WIN32:
+        if command[0].endswith(".py"):
+            with open(command[0], "r") as script_file:
+                if not userxbit(command[0]):
+                    # Inform user that Python scripts need x-bit and then stop
+                    logging.error(T('Python script "%s" does not have execute (+x) permission set'), command[0])
+                    raise IOError
+                elif script_file.read(2) != "#!":
+                    # No shebang (#!) defined, add default python
+                    command.insert(0, "python")
+
+        if sabnzbd.newsunpack.IONICE_COMMAND and cfg.ionice():
+            ionice = cfg.ionice().split()
+            command = ionice + command
+            command.insert(0, sabnzbd.newsunpack.IONICE_COMMAND)
+        if sabnzbd.newsunpack.NICE_COMMAND and cfg.nice():
+            nice = cfg.nice().split()
+            command = nice + command
+            command.insert(0, sabnzbd.newsunpack.NICE_COMMAND)
+        creationflags = 0
+        startupinfo = None
+    else:
+        # For Windows we always need to add python interpreter
+        if command[0].endswith(".py"):
+            command.insert(0, "python.exe")
+        if flatten_command:
+            command = list2cmdline(command)
+        # On some Windows platforms we need to supress a quick pop-up of the command window
+        startupinfo = subprocess.STARTUPINFO()
+        startupinfo.dwFlags = win32process.STARTF_USESHOWWINDOW
+        startupinfo.wShowWindow = win32con.SW_HIDE
+        creationflags = WIN_SCHED_PRIOS[cfg.win_process_prio()]
+
+    # Set the basic Popen arguments
+    popen_kwargs = {
+        "stdin": subprocess.PIPE,
+        "stdout": subprocess.PIPE,
+        "stderr": subprocess.STDOUT,
+        "startupinfo": startupinfo,
+        "creationflags": creationflags,
+    }
+    # Update with the supplied ones
+    popen_kwargs.update(kwargs)
+
+    # Run the command
+    logging.info("[%s] Running external command: %s", caller_name(), command)
+    logging.debug("Popen arguments: %s", popen_kwargs)
+    return subprocess.Popen(command, **popen_kwargs)
+
+
+def run_command(cmd: List[str], **kwargs):
+    """ Run simple external command and return output as a string. """
+    with build_and_run_command(cmd, **kwargs) as p:
+        txt = platform_btou(p.stdout.read())
+        p.wait()
+    return txt
