@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2020 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2021 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -28,12 +28,20 @@ import time
 import zlib
 import shutil
 import functools
-from subprocess import Popen
 
 import sabnzbd
 from sabnzbd.encoding import platform_btou, correct_unknown_encoding, ubtou
 import sabnzbd.utils.rarfile as rarfile
-from sabnzbd.misc import format_time_string, find_on_path, int_conv, get_all_passwords, calc_age, cmp, caller_name
+from sabnzbd.misc import (
+    format_time_string,
+    find_on_path,
+    int_conv,
+    get_all_passwords,
+    calc_age,
+    cmp,
+    run_command,
+    build_and_run_command,
+)
 from sabnzbd.filesystem import (
     make_script_path,
     real_path,
@@ -47,28 +55,13 @@ from sabnzbd.filesystem import (
     setname_from_path,
     get_ext,
     get_filename,
+    same_file,
 )
+from sabnzbd.nzbstuff import NzbObject, NzbFile
 from sabnzbd.sorting import SeriesSorter
 import sabnzbd.cfg as cfg
 from sabnzbd.constants import Status
 
-if sabnzbd.WIN32:
-    try:
-        import win32api
-        import win32con
-        import win32process
-
-        # Define scheduling priorities
-        WIN_SCHED_PRIOS = {
-            1: win32process.IDLE_PRIORITY_CLASS,
-            2: win32process.BELOW_NORMAL_PRIORITY_CLASS,
-            3: win32process.NORMAL_PRIORITY_CLASS,
-            4: win32process.ABOVE_NORMAL_PRIORITY_CLASS,
-        }
-    except ImportError:
-        pass
-
-
 # Regex globals
 RAR_RE = re.compile(r"\.(?P<ext>part\d*\.rar|rar|r\d\d|s\d\d|t\d\d|u\d\d|v\d\d|\d\d\d?\d)$", re.I)
 RAR_RE_V3 = re.compile(r"\.(?P<ext>part\d*)$", re.I)
@@ -124,7 +117,14 @@ def find_programs(curdir):
         if not sabnzbd.newsunpack.PAR2_COMMAND:
             sabnzbd.newsunpack.PAR2_COMMAND = find_on_path("par2")
         if not sabnzbd.newsunpack.RAR_COMMAND:
-            sabnzbd.newsunpack.RAR_COMMAND = find_on_path(("unrar", "rar", "unrar3", "rar3",))
+            sabnzbd.newsunpack.RAR_COMMAND = find_on_path(
+                (
+                    "unrar",
+                    "rar",
+                    "unrar3",
+                    "rar3",
+                )
+            )
         sabnzbd.newsunpack.NICE_COMMAND = find_on_path("nice")
         sabnzbd.newsunpack.IONICE_COMMAND = find_on_path("ionice")
         if not sabnzbd.newsunpack.ZIP_COMMAND:
@@ -169,7 +169,7 @@ ENV_NZO_FIELDS = [
 ]
 
 
-def external_processing(extern_proc, nzo, complete_dir, nicename, status):
+def external_processing(extern_proc, nzo: NzbObject, complete_dir, nicename, status):
     """ Run a user postproc script, return console output and exit value """
     failure_url = nzo.nzo_info.get("failure", "")
     # Items can be bool or null, causing POpen to fail
@@ -186,7 +186,7 @@ def external_processing(extern_proc, nzo, complete_dir, nicename, status):
     ]
 
     # Add path to original NZB
-    nzb_paths = globber_full(nzo.workpath, "*.gz")
+    nzb_paths = globber_full(nzo.admin_path, "*.gz")
 
     # Fields not in the NZO directly
     extra_env_fields = {
@@ -200,31 +200,8 @@ def external_processing(extern_proc, nzo, complete_dir, nicename, status):
     }
 
     try:
-        stup, need_shell, command, creationflags = build_command(command)
-        env = create_env(nzo, extra_env_fields)
-
-        logging.info(
-            "Running external script %s(%s, %s, %s, %s, %s, %s, %s, %s)",
-            extern_proc,
-            complete_dir,
-            nzo.filename,
-            nicename,
-            "",
-            nzo.cat,
-            nzo.group,
-            status,
-            failure_url,
-        )
-        p = Popen(
-            command,
-            shell=need_shell,
-            stdin=subprocess.PIPE,
-            stdout=subprocess.PIPE,
-            stderr=subprocess.STDOUT,
-            startupinfo=stup,
-            env=env,
-            creationflags=creationflags,
-        )
+        p = build_and_run_command(command, env=create_env(nzo, extra_env_fields))
+        sabnzbd.PostProcessor.external_process = p
 
         # Follow the output, so we can abort it
         proc = p.stdout
@@ -241,14 +218,6 @@ def external_processing(extern_proc, nzo, complete_dir, nicename, status):
 
             # Show current line in history
             nzo.set_action_line(T("Running script"), line)
-
-            # Check if we should still continue
-            if not nzo.pp_active:
-                p.kill()
-                lines.append(T("PostProcessing was aborted (%s)") % T("Script"))
-                # Print at least what we got
-                output = "\n".join(lines)
-                return output, 1
     except:
         logging.debug("Failed script %s, Traceback: ", extern_proc, exc_info=True)
         return "Cannot run script %s\r\n" % extern_proc, -1
@@ -258,34 +227,9 @@ def external_processing(extern_proc, nzo, complete_dir, nicename, status):
     return output, ret
 
 
-def external_script(script, p1, p2, p3=None, p4=None):
-    """ Run a user script with two parameters, return console output and exit value """
-    command = [script, p1, p2, p3, p4]
-
-    try:
-        stup, need_shell, command, creationflags = build_command(command)
-        env = create_env()
-        logging.info("Running user script %s(%s, %s)", script, p1, p2)
-        p = Popen(
-            command,
-            shell=need_shell,
-            stdin=subprocess.PIPE,
-            stdout=subprocess.PIPE,
-            stderr=subprocess.STDOUT,
-            startupinfo=stup,
-            env=env,
-            creationflags=creationflags,
-        )
-    except:
-        logging.debug("Failed script %s, Traceback: ", script, exc_info=True)
-        return "Cannot run script %s\r\n" % script, -1
-
-    output = platform_btou(p.stdout.read())
-    ret = p.wait()
-    return output, ret
-
-
-def unpack_magic(nzo, workdir, workdir_complete, dele, one_folder, joinables, zips, rars, sevens, ts, depth=0):
+def unpack_magic(
+    nzo: NzbObject, workdir, workdir_complete, dele, one_folder, joinables, zips, rars, sevens, ts, depth=0
+):
     """ Do a recursive unpack from all archives in 'workdir' to 'workdir_complete' """
     if depth > 5:
         logging.warning(T("Unpack nesting too deep [%s]"), nzo.final_name)
@@ -433,9 +377,9 @@ def get_seq_number(name):
         return 0
 
 
-def file_join(nzo, workdir, workdir_complete, delete, joinables):
-    """ Join and joinable files in 'workdir' to 'workdir_complete' and
-        when successful, delete originals
+def file_join(nzo: NzbObject, workdir, workdir_complete, delete, joinables):
+    """Join and joinable files in 'workdir' to 'workdir_complete' and
+    when successful, delete originals
     """
     newfiles = []
     bufsize = 24 * 1024 * 1024
@@ -524,10 +468,10 @@ def file_join(nzo, workdir, workdir_complete, delete, joinables):
 ##############################################################################
 # (Un)Rar Functions
 ##############################################################################
-def rar_unpack(nzo, workdir, workdir_complete, delete, one_folder, rars):
-    """ Unpack multiple sets 'rars' of RAR files from 'workdir' to 'workdir_complete.
-        When 'delete' is set, originals will be deleted.
-        When 'one_folder' is set, all files will be in a single folder
+def rar_unpack(nzo: NzbObject, workdir, workdir_complete, delete, one_folder, rars):
+    """Unpack multiple sets 'rars' of RAR files from 'workdir' to 'workdir_complete.
+    When 'delete' is set, originals will be deleted.
+    When 'one_folder' is set, all files will be in a single folder
     """
     newfiles = extracted_files = []
     rar_sets = {}
@@ -587,10 +531,6 @@ def rar_unpack(nzo, workdir, workdir_complete, delete, one_folder, rars):
                 fail, newfiles, rars = rar_extract(
                     rarpath, len(rar_sets[rar_set]), one_folder, nzo, rar_set, extraction_path
                 )
-                # Was it aborted?
-                if not nzo.pp_active:
-                    fail = True
-                    break
                 success = not fail
             except:
                 success = False
@@ -647,10 +587,10 @@ def rar_unpack(nzo, workdir, workdir_complete, delete, one_folder, rars):
     return fail, extracted_files
 
 
-def rar_extract(rarfile_path, numrars, one_folder, nzo, setname, extraction_path):
-    """ Unpack single rar set 'rarfile' to 'extraction_path',
-        with password tries
-        Return fail==0(ok)/fail==1(error)/fail==2(wrong password), new_files, rars
+def rar_extract(rarfile_path, numrars, one_folder, nzo: NzbObject, setname, extraction_path):
+    """Unpack single rar set 'rarfile' to 'extraction_path',
+    with password tries
+    Return fail==0(ok)/fail==1(error)/fail==2(wrong password), new_files, rars
     """
     fail = 0
     new_files = None
@@ -674,9 +614,9 @@ def rar_extract(rarfile_path, numrars, one_folder, nzo, setname, extraction_path
     return fail, new_files, rars
 
 
-def rar_extract_core(rarfile_path, numrars, one_folder, nzo, setname, extraction_path, password):
-    """ Unpack single rar set 'rarfile_path' to 'extraction_path'
-        Return fail==0(ok)/fail==1(error)/fail==2(wrong password)/fail==3(crc-error), new_files, rars
+def rar_extract_core(rarfile_path, numrars, one_folder, nzo: NzbObject, setname, extraction_path, password):
+    """Unpack single rar set 'rarfile_path' to 'extraction_path'
+    Return fail==0(ok)/fail==1(error)/fail==2(wrong password)/fail==3(crc-error), new_files, rars
     """
     start = time.time()
 
@@ -701,22 +641,25 @@ def rar_extract_core(rarfile_path, numrars, one_folder, nzo, setname, extraction
         rename = "-or"  # Auto renaming
 
     if sabnzbd.WIN32:
-        # For Unrar to support long-path, we need to cricumvent Python's list2cmdline
+        # For Unrar to support long-path, we need to circumvent Python's list2cmdline
         # See: https://github.com/sabnzbd/sabnzbd/issues/1043
+        # The -scf forces the output to be UTF8
         command = [
             "%s" % RAR_COMMAND,
             action,
             "-idp",
+            "-scf",
             overwrite,
             rename,
             "-ai",
             password_command,
-            "%s" % clip_path(rarfile_path),
+            rarfile_path,
             "%s\\" % long_path(extraction_path),
         ]
 
     elif RAR_PROBLEM:
-        # Use only oldest options (specifically no "-or")
+        # Use only oldest options, specifically no "-or" or "-scf"
+        # Luckily platform_btou has a fallback for non-UTF-8
         command = [
             "%s" % RAR_COMMAND,
             action,
@@ -728,10 +671,12 @@ def rar_extract_core(rarfile_path, numrars, one_folder, nzo, setname, extraction
         ]
     else:
         # Don't use "-ai" (not needed for non-Windows)
+        # The -scf forces the output to be UTF8
         command = [
             "%s" % RAR_COMMAND,
             action,
             "-idp",
+            "-scf",
             overwrite,
             rename,
             password_command,
@@ -742,20 +687,10 @@ def rar_extract_core(rarfile_path, numrars, one_folder, nzo, setname, extraction
     if cfg.ignore_unrar_dates():
         command.insert(3, "-tsm-")
 
-    stup, need_shell, command, creationflags = build_command(command, flatten_command=True)
-
     # Get list of all the volumes part of this set
     logging.debug("Analyzing rar file ... %s found", rarfile.is_rarfile(rarfile_path))
-    logging.debug("Running unrar %s", command)
-    p = Popen(
-        command,
-        shell=need_shell,
-        stdin=subprocess.PIPE,
-        stdout=subprocess.PIPE,
-        stderr=subprocess.STDOUT,
-        startupinfo=stup,
-        creationflags=creationflags,
-    )
+    p = build_and_run_command(command, flatten_command=True)
+    sabnzbd.PostProcessor.external_process = p
 
     proc = p.stdout
     if p.stdin:
@@ -776,15 +711,6 @@ def rar_extract_core(rarfile_path, numrars, one_folder, nzo, setname, extraction
         if not line:
             break
 
-        # Check if we should still continue
-        if not nzo.pp_active:
-            p.kill()
-            msg = T("PostProcessing was aborted (%s)") % T("Unpack")
-            nzo.fail_msg = msg
-            nzo.set_unpack_info("Unpack", msg, setname)
-            nzo.status = Status.FAILED
-            return fail, (), ()
-
         line = line.strip()
         lines.append(line)
 
@@ -920,9 +846,9 @@ def rar_extract_core(rarfile_path, numrars, one_folder, nzo, setname, extraction
 ##############################################################################
 # (Un)Zip Functions
 ##############################################################################
-def unzip(nzo, workdir, workdir_complete, delete, one_folder, zips):
-    """ Unpack multiple sets 'zips' of ZIP files from 'workdir' to 'workdir_complete.
-        When 'delete' is ste, originals will be deleted.
+def unzip(nzo: NzbObject, workdir, workdir_complete, delete, one_folder, zips):
+    """Unpack multiple sets 'zips' of ZIP files from 'workdir' to 'workdir_complete.
+    When 'delete' is ste, originals will be deleted.
     """
 
     try:
@@ -988,32 +914,19 @@ def ZIP_Extract(zipfile, extraction_path, one_folder):
     if one_folder or cfg.flat_unpack():
         command.insert(3, "-j")  # Unpack without folders
 
-    stup, need_shell, command, creationflags = build_command(command)
-    logging.debug("Starting unzip: %s", command)
-    p = Popen(
-        command,
-        shell=need_shell,
-        stdin=subprocess.PIPE,
-        stdout=subprocess.PIPE,
-        stderr=subprocess.STDOUT,
-        startupinfo=stup,
-        creationflags=creationflags,
-    )
-
+    p = build_and_run_command(command)
     output = platform_btou(p.stdout.read())
-    logging.debug("unzip output: \n%s", output)
-
     ret = p.wait()
-
+    logging.debug("unzip output: \n%s", output)
     return ret
 
 
 ##############################################################################
 # 7Zip Functions
 ##############################################################################
-def unseven(nzo, workdir, workdir_complete, delete, one_folder, sevens):
-    """ Unpack multiple sets '7z' of 7Zip files from 'workdir' to 'workdir_complete.
-        When 'delete' is set, originals will be deleted.
+def unseven(nzo: NzbObject, workdir, workdir_complete, delete, one_folder, sevens):
+    """Unpack multiple sets '7z' of 7Zip files from 'workdir' to 'workdir_complete.
+    When 'delete' is set, originals will be deleted.
     """
     i = 0
     unseven_failed = False
@@ -1059,9 +972,9 @@ def unseven(nzo, workdir, workdir_complete, delete, one_folder, sevens):
     return unseven_failed, new_files
 
 
-def seven_extract(nzo, sevenset, extensions, extraction_path, one_folder, delete):
-    """ Unpack single set 'sevenset' to 'extraction_path', with password tries
-        Return fail==0(ok)/fail==1(error)/fail==2(wrong password), new_files, sevens
+def seven_extract(nzo: NzbObject, sevenset, extensions, extraction_path, one_folder, delete):
+    """Unpack single set 'sevenset' to 'extraction_path', with password tries
+    Return fail==0(ok)/fail==1(error)/fail==2(wrong password), new_files, sevens
     """
     # Before we start, make sure the 7z binary SEVEN_COMMAND is defined
     if not SEVEN_COMMAND:
@@ -1093,8 +1006,8 @@ def seven_extract(nzo, sevenset, extensions, extraction_path, one_folder, delete
 
 
 def seven_extract_core(sevenset, extensions, extraction_path, one_folder, delete, password):
-    """ Unpack single 7Z set 'sevenset' to 'extraction_path'
-        Return fail==0(ok)/fail==1(error)/fail==2(wrong password), new_files, message
+    """Unpack single 7Z set 'sevenset' to 'extraction_path'
+    Return fail==0(ok)/fail==1(error)/fail==2(wrong password), new_files, message
     """
     if one_folder:
         method = "e"  # Unpack without folders
@@ -1127,19 +1040,8 @@ def seven_extract_core(sevenset, extensions, extraction_path, one_folder, delete
     orig_dir_content = listdir_full(extraction_path)
 
     command = [SEVEN_COMMAND, method, "-y", overwrite, parm, case, password, "-o%s" % extraction_path, name]
-
-    stup, need_shell, command, creationflags = build_command(command)
-    logging.debug("Starting 7za: %s", command)
-    p = Popen(
-        command,
-        shell=need_shell,
-        stdin=subprocess.PIPE,
-        stdout=subprocess.PIPE,
-        stderr=subprocess.STDOUT,
-        startupinfo=stup,
-        creationflags=creationflags,
-    )
-
+    p = build_and_run_command(command)
+    sabnzbd.PostProcessor.external_process = p
     output = platform_btou(p.stdout.read())
     logging.debug("7za output: %s", output)
 
@@ -1178,7 +1080,7 @@ def seven_extract_core(sevenset, extensions, extraction_path, one_folder, delete
 ##############################################################################
 # PAR2 Functions
 ##############################################################################
-def par2_repair(parfile_nzf, nzo, workdir, setname, single):
+def par2_repair(parfile_nzf: NzbFile, nzo: NzbObject, workdir, setname, single):
     """ Try to repair a set, return readd or correctness """
     # Check if file exists, otherwise see if another is done
     parfile_path = os.path.join(workdir, parfile_nzf.filename)
@@ -1304,7 +1206,7 @@ _RE_LOADING_PAR2 = re.compile(r'Loading "([^"]+)"\.')
 _RE_LOADED_PAR2 = re.compile(r"Loaded (\d+) new packets")
 
 
-def PAR_Verify(parfile, nzo, setname, joinables, single=False):
+def PAR_Verify(parfile, nzo: NzbObject, setname, joinables, single=False):
     """ Run par2 on par-set """
     used_joinables = []
     used_for_repair = []
@@ -1335,7 +1237,7 @@ def PAR_Verify(parfile, nzo, setname, joinables, single=False):
     # Or the one that complains about basepath
     # Only if we're not doing multicore
     if not sabnzbd.WIN32 and not sabnzbd.DARWIN:
-        par2text = run_simple([command[0], "-h"])
+        par2text = run_command([command[0], "-h"])
         if "No data skipping" in par2text:
             logging.info("Detected par2cmdline version that skips blocks, adding -N parameter")
             command.insert(2, "-N")
@@ -1344,24 +1246,14 @@ def PAR_Verify(parfile, nzo, setname, joinables, single=False):
             command.insert(2, "-B")
             command.insert(3, parfolder)
 
-    stup, need_shell, command, creationflags = build_command(command)
-
     # par2multicore wants to see \\.\ paths on Windows
     # See: https://github.com/sabnzbd/sabnzbd/pull/771
     if sabnzbd.WIN32:
         command = [clip_path(x) if x.startswith("\\\\?\\") else x for x in command]
 
     # Run the external command
-    logging.info("Starting par2: %s", command)
-    p = Popen(
-        command,
-        shell=need_shell,
-        stdin=subprocess.PIPE,
-        stdout=subprocess.PIPE,
-        stderr=subprocess.STDOUT,
-        startupinfo=stup,
-        creationflags=creationflags,
-    )
+    p = build_and_run_command(command)
+    sabnzbd.PostProcessor.external_process = p
     proc = p.stdout
 
     if p.stdin:
@@ -1397,16 +1289,6 @@ def PAR_Verify(parfile, nzo, setname, joinables, single=False):
         line = linebuf.strip()
         linebuf = ""
 
-        # Check if we should still continue
-        if not nzo.pp_active:
-            p.kill()
-            msg = T("PostProcessing was aborted (%s)") % T("Repair")
-            nzo.fail_msg = msg
-            nzo.set_unpack_info("Repair", msg, setname)
-            nzo.status = Status.FAILED
-            readd = False
-            break
-
         # Skip empty lines
         if line == "":
             continue
@@ -1450,7 +1332,7 @@ def PAR_Verify(parfile, nzo, setname, joinables, single=False):
                     block_table[nzf.blocks] = nzf
 
             if block_table:
-                nzf = block_table[min(block_table.keys())]
+                nzf = block_table[min(block_table)]
                 logging.info("Found new par2file %s", nzf.filename)
 
                 # Move from extrapar list to files to be downloaded
@@ -1636,7 +1518,7 @@ def PAR_Verify(parfile, nzo, setname, joinables, single=False):
 _RE_FILENAME = re.compile(r'"([^"]+)"')
 
 
-def MultiPar_Verify(parfile, nzo, setname, joinables, single=False):
+def MultiPar_Verify(parfile, nzo: NzbObject, setname, joinables, single=False):
     """ Run par2 on par-set """
     parfolder = os.path.split(parfile)[0]
     used_joinables = []
@@ -1646,9 +1528,10 @@ def MultiPar_Verify(parfile, nzo, setname, joinables, single=False):
     nzo.status = Status.VERIFYING
     start = time.time()
 
-    # Caching of verification implemented by adding:
+    # Caching of verification implemented by adding -vs/-vd
+    # Force output of utf-8 by adding -uo
     # But not really required due to prospective-par2
-    command = [str(MULTIPAR_COMMAND), "r", "-vs2", "-vd%s" % parfolder, parfile]
+    command = [str(MULTIPAR_COMMAND), "r", "-uo", "-vs2", "-vd%s" % parfolder, parfile]
 
     # Check if there are maybe par2cmdline/par2tbb commands supplied
     if "-t" in cfg.par_option() or "-p" in cfg.par_option():
@@ -1671,31 +1554,20 @@ def MultiPar_Verify(parfile, nzo, setname, joinables, single=False):
         wildcard = setname + "*"
     command.append(os.path.join(parfolder, wildcard))
 
-    stup, need_shell, command, creationflags = build_command(command)
-    logging.info("Starting MultiPar: %s", command)
-
-    lines = []
-    p = Popen(
-        command,
-        shell=need_shell,
-        stdin=subprocess.PIPE,
-        stdout=subprocess.PIPE,
-        stderr=subprocess.STDOUT,
-        startupinfo=stup,
-        creationflags=creationflags,
-    )
-
+    # Run MultiPar
+    p = build_and_run_command(command)
+    sabnzbd.PostProcessor.external_process = p
     proc = p.stdout
-
     if p.stdin:
         p.stdin.close()
 
     # Set up our variables
+    lines = []
     datafiles = []
     renames = {}
     reconstructed = []
 
-    linebuf = ""
+    linebuf = b""
     finished = 0
     readd = False
 
@@ -1711,27 +1583,17 @@ def MultiPar_Verify(parfile, nzo, setname, joinables, single=False):
 
     # Loop over the output, whee
     while 1:
-        char = platform_btou(proc.read(1))
+        char = proc.read(1)
         if not char:
             break
 
         # Line not complete yet
-        if char not in ("\n", "\r"):
+        if char not in (b"\n", b"\r"):
             linebuf += char
             continue
 
-        line = linebuf.strip()
-        linebuf = ""
-
-        # Check if we should still continue
-        if not nzo.pp_active:
-            p.kill()
-            msg = T("PostProcessing was aborted (%s)") % T("Repair")
-            nzo.fail_msg = msg
-            nzo.set_unpack_info("Repair", msg, setname)
-            nzo.status = Status.FAILED
-            readd = False
-            break
+        line = ubtou(linebuf).strip()
+        linebuf = b""
 
         # Skip empty lines
         if line == "":
@@ -1762,7 +1624,7 @@ def MultiPar_Verify(parfile, nzo, setname, joinables, single=False):
                     block_table[nzf.blocks] = nzf
 
             if block_table:
-                nzf = block_table[min(block_table.keys())]
+                nzf = block_table[min(block_table)]
                 logging.info("Found new par2file %s", nzf.filename)
 
                 # Move from extrapar list to files to be downloaded
@@ -1992,7 +1854,7 @@ def MultiPar_Verify(parfile, nzo, setname, joinables, single=False):
             verifynum += 1
             nzo.set_action_line(T("Verifying repair"), "%02d/%02d" % (verifynum, verifytotal))
 
-        elif line.startswith("Failed to repair"):
+        elif line.startswith("Failed to repair") and not readd:
             # Unknown repair problem
             msg = T("Repairing failed, %s") % line
             nzo.fail_msg = msg
@@ -2028,9 +1890,9 @@ def MultiPar_Verify(parfile, nzo, setname, joinables, single=False):
 
 
 def create_env(nzo=None, extra_env_fields={}):
-    """ Modify the environment for pp-scripts with extra information
-        OSX: Return copy of environment without PYTHONPATH and PYTHONHOME
-        other: return None
+    """Modify the environment for pp-scripts with extra information
+    macOS: Return copy of environment without PYTHONPATH and PYTHONHOME
+    other: return None
     """
     env = os.environ.copy()
 
@@ -2087,76 +1949,10 @@ def create_env(nzo=None, extra_env_fields={}):
     return env
 
 
-def userxbit(filename):
-    # Returns boolean if the x-bit for user is set on the given file
-    # This is a workaround: os.access(filename, os.X_OK) does not work on certain mounted file systems
-    # Does not work on Windows, but it is not called on Windows
-
-    # rwx rwx rwx
-    # 876 543 210      # we want bit 6 from the right, counting from 0
-    userxbit = 1 << 6  # bit 6
-    rwxbits = os.stat(filename)[0]  # the first element of os.stat() is "mode"
-    # do logical AND, check if it is not 0:
-    xbitset = (rwxbits & userxbit) > 0
-    return xbitset
-
-
-def build_command(command, flatten_command=False):
-    """ Prepare list from running an external program
-        On Windows we need to run our own list2cmdline for Unrar
-    """
-    # command[0] should be set, and thus not None
-    if not command[0]:
-        logging.error(T("[%s] The command in build_command is undefined."), caller_name())
-        raise IOError
-
-    if not sabnzbd.WIN32:
-        if command[0].endswith(".py"):
-            with open(command[0], "r") as script_file:
-                if not userxbit(command[0]):
-                    # Inform user that Python scripts need x-bit and then stop
-                    logging.error(T('Python script "%s" does not have execute (+x) permission set'), command[0])
-                    raise IOError
-                elif script_file.read(2) != "#!":
-                    # No shebang (#!) defined, add default python
-                    command.insert(0, "python")
-
-        if IONICE_COMMAND and cfg.ionice().strip():
-            lst = cfg.ionice().split()
-            lst.reverse()
-            for arg in lst:
-                command.insert(0, arg)
-            command.insert(0, IONICE_COMMAND)
-        if NICE_COMMAND and cfg.nice().strip():
-            lst = cfg.nice().split()
-            lst.reverse()
-            for arg in lst:
-                command.insert(0, arg)
-            command.insert(0, NICE_COMMAND)
-        need_shell = False
-        stup = None
-        creationflags = 0
-
-    else:
-        # For Windows we always need to add python interpreter
-        if command[0].endswith(".py"):
-            command.insert(0, "python")
-
-        need_shell = os.path.splitext(command[0])[1].lower() not in (".exe", ".com")
-        stup = subprocess.STARTUPINFO()
-        stup.dwFlags = win32process.STARTF_USESHOWWINDOW
-        stup.wShowWindow = win32con.SW_HIDE
-        creationflags = WIN_SCHED_PRIOS[cfg.win_process_prio()]
-
-        if need_shell or flatten_command:
-            command = list2cmdline(command)
-
-    return stup, need_shell, command, creationflags
-
-
 def rar_volumelist(rarfile_path, password, known_volumes):
-    """ Extract volumes that are part of this rarset
-        and merge them with existing list, removing duplicates
+    """List volumes that are part of this rarset
+    and merge them with parsed paths list, removing duplicates.
+    We assume RarFile is right and use parsed paths as backup.
     """
     # UnRar is required to read some RAR files
     # RarFile can fail in special cases
@@ -2175,12 +1971,12 @@ def rar_volumelist(rarfile_path, password, known_volumes):
         zf_volumes = []
 
     # Remove duplicates
-    known_volumes_base = [os.path.basename(vol) for vol in known_volumes]
-    for zf_volume in zf_volumes:
-        if os.path.basename(zf_volume) not in known_volumes_base:
+    zf_volumes_base = [os.path.basename(vol) for vol in zf_volumes]
+    for known_volume in known_volumes:
+        if os.path.basename(known_volume) not in zf_volumes_base:
             # Long-path notation just to be sure
-            known_volumes.append(long_path(zf_volume))
-    return known_volumes
+            zf_volumes.append(long_path(known_volume))
+    return zf_volumes
 
 
 # Sort the various RAR filename formats properly :\
@@ -2200,9 +1996,9 @@ def rar_sort(a, b):
 
 
 def build_filelists(workdir, workdir_complete=None, check_both=False, check_rar=True):
-    """ Build filelists, if workdir_complete has files, ignore workdir.
-        Optionally scan both directories.
-        Optionally test content to establish RAR-ness
+    """Build filelists, if workdir_complete has files, ignore workdir.
+    Optionally scan both directories.
+    Optionally test content to establish RAR-ness
     """
     sevens, joinables, zips, rars, ts, filelist = ([], [], [], [], [], [])
 
@@ -2282,7 +2078,14 @@ def quick_check_set(set, nzo):
             if nzf.md5sum == md5pack[file]:
                 try:
                     logging.debug("Quick-check will rename %s to %s", nzf.filename, file)
-                    renamer(os.path.join(nzo.downpath, nzf.filename), os.path.join(nzo.downpath, file))
+
+                    # Note: file can and is allowed to be in a subdirectory.
+                    # Subdirectories in par2 always contain "/", not "\"
+                    renamer(
+                        os.path.join(nzo.download_path, nzf.filename),
+                        os.path.join(nzo.download_path, file),
+                        create_local_directories=True,
+                    )
                     renames[file] = nzf.filename
                     nzf.filename = file
                     result &= True
@@ -2309,15 +2112,15 @@ def quick_check_set(set, nzo):
 
 
 def unrar_check(rar):
-    """ Return version number of unrar, where "5.01" returns 501
-        Also return whether an original version is found
-        (version, original)
+    """Return version number of unrar, where "5.01" returns 501
+    Also return whether an original version is found
+    (version, original)
     """
     version = 0
     original = ""
     if rar:
         try:
-            version = run_simple(rar)
+            version = run_command([rar])
         except:
             return version, original
         original = "Alexander Roshal" in version
@@ -2332,7 +2135,7 @@ def unrar_check(rar):
 def par2_mt_check(par2_path):
     """ Detect if we have multicore par2 variants """
     try:
-        par2_version = run_simple([par2_path, "-h"])
+        par2_version = run_command([par2_path, "-h"])
         # Look for a threads option
         if "-t<" in par2_version:
             return True
@@ -2385,7 +2188,7 @@ def is_sfv_file(myfile):
     return sfv_info_line_counter >= 1
 
 
-def sfv_check(sfvs, nzo, workdir):
+def sfv_check(sfvs, nzo: NzbObject, workdir):
     """ Verify files using SFV files """
     # Update status
     nzo.status = Status.VERIFYING
@@ -2442,7 +2245,7 @@ def sfv_check(sfvs, nzo, workdir):
             if nzf.filename in calculated_crc32 and calculated_crc32[nzf.filename] == sfv_parse_results[file]:
                 try:
                     logging.debug("SFV-check will rename %s to %s", nzf.filename, file)
-                    renamer(os.path.join(nzo.downpath, nzf.filename), os.path.join(nzo.downpath, file))
+                    renamer(os.path.join(nzo.download_path, nzf.filename), os.path.join(nzo.download_path, file))
                     renames[file] = nzf.filename
                     nzf.filename = file
                     result &= True
@@ -2508,9 +2311,9 @@ def analyse_show(name):
     return show_name, info.get("season_num", ""), info.get("episode_num", ""), info.get("ep_name", "")
 
 
-def pre_queue(nzo, pp, cat):
-    """ Run pre-queue script (if any) and process results.
-        pp and cat are supplied seperate since they can change.
+def pre_queue(nzo: NzbObject, pp, cat):
+    """Run pre-queue script (if any) and process results.
+    pp and cat are supplied seperate since they can change.
     """
 
     def fix(p):
@@ -2546,19 +2349,7 @@ def pre_queue(nzo, pp, cat):
         }
 
         try:
-            stup, need_shell, command, creationflags = build_command(command)
-            env = create_env(nzo, extra_env_fields)
-            logging.info("Running pre-queue script %s", command)
-            p = Popen(
-                command,
-                shell=need_shell,
-                stdin=subprocess.PIPE,
-                stdout=subprocess.PIPE,
-                stderr=subprocess.STDOUT,
-                startupinfo=stup,
-                env=env,
-                creationflags=creationflags,
-            )
+            p = build_and_run_command(command, env=create_env(nzo, extra_env_fields))
         except:
             logging.debug("Failed script %s, Traceback: ", script_path, exc_info=True)
             return values
@@ -2584,17 +2375,6 @@ def pre_queue(nzo, pp, cat):
     return values
 
 
-def list2cmdline(lst):
-    """ convert list to a cmd.exe-compatible command string """
-    nlst = []
-    for arg in lst:
-        if not arg:
-            nlst.append('""')
-        else:
-            nlst.append('"%s"' % arg)
-    return " ".join(nlst)
-
-
 def is_sevenfile(path):
     """ Return True if path has proper extension and 7Zip is installed """
     return SEVEN_COMMAND and os.path.splitext(path)[1].lower() == ".7z"
@@ -2611,20 +2391,8 @@ class SevenZip:
         names = []
         # Future extension: use '-sccUTF-8' to get names in UTF8 encoding
         command = [SEVEN_COMMAND, "l", "-p", "-y", "-slt", self.path]
-        stup, need_shell, command, creationflags = build_command(command)
+        output = run_command(command)
 
-        p = Popen(
-            command,
-            shell=need_shell,
-            stdin=subprocess.PIPE,
-            stdout=subprocess.PIPE,
-            stderr=subprocess.STDOUT,
-            startupinfo=stup,
-            creationflags=creationflags,
-        )
-
-        output = platform_btou(p.stdout.read())
-        _ = p.wait()
         re_path = re.compile("^Path = (.+)")
         for line in output.split("\n"):
             m = re_path.search(line)
@@ -2638,30 +2406,9 @@ class SevenZip:
     def read(self, name):
         """ Read named file from 7Zip and return data """
         command = [SEVEN_COMMAND, "e", "-p", "-y", "-so", self.path, name]
-        stup, need_shell, command, creationflags = build_command(command)
-
         # Ignore diagnostic output, otherwise it will be appended to content
-        p = Popen(
-            command,
-            shell=need_shell,
-            stdin=subprocess.PIPE,
-            stdout=subprocess.PIPE,
-            stderr=subprocess.DEVNULL,
-            startupinfo=stup,
-            creationflags=creationflags,
-        )
-        output = platform_btou(p.stdout.read())
-        _ = p.wait()
-        return output
+        return run_command(command, stderr=subprocess.DEVNULL)
 
     def close(self):
         """ Close file """
         pass
-
-
-def run_simple(cmd):
-    """ Run simple external command and return output """
-    with Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE) as p:
-        txt = platform_btou(p.stdout.read())
-        p.wait()
-    return txt
