@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2020 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2021 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -75,12 +75,12 @@ def win_shutdown():
 
 
 ##############################################################################
-# Power management for OSX
+# Power management for macOS
 ##############################################################################
 
 
 def osx_shutdown():
-    """ Shutdown OSX system, never returns """
+    """ Shutdown macOS system, never returns """
     try:
         subprocess.call(["osascript", "-e", 'tell app "System Events" to shut down'])
     except:
@@ -90,7 +90,7 @@ def osx_shutdown():
 
 
 def osx_standby():
-    """ Make OSX system sleep, returns after wakeup """
+    """ Make macOS system sleep, returns after wakeup """
     try:
         subprocess.call(["osascript", "-e", 'tell app "System Events" to sleep'])
         time.sleep(10)
@@ -100,7 +100,7 @@ def osx_standby():
 
 
 def osx_hibernate():
-    """ Make OSX system sleep, returns after wakeup """
+    """ Make macOS system sleep, returns after wakeup """
     osx_standby()
 
 
@@ -196,7 +196,7 @@ def linux_shutdown():
                 else:
                     logging.info("DBus does not support Stop (shutdown)")
     except dbus.exceptions.DBusException as msg:
-        logging.error("Received a DBus exception %s", msg)
+        logging.error(T("Received a DBus exception %s"), msg)
     os._exit(0)
 
 
@@ -226,7 +226,7 @@ def linux_hibernate():
                     logging.info("DBus does not support Hibernate")
         time.sleep(10)
     except dbus.exceptions.DBusException as msg:
-        logging.error("Received a DBus exception %s", msg)
+        logging.error(T("Received a DBus exception %s"), msg)
 
 
 def linux_standby():
@@ -255,4 +255,4 @@ def linux_standby():
                     logging.info("DBus does not support Suspend (standby)")
         time.sleep(10)
     except dbus.exceptions.DBusException as msg:
-        logging.error("Received a DBus exception %s", msg)
+        logging.error(T("Received a DBus exception %s"), msg)
