@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2020 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2021 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -24,6 +24,7 @@ import logging
 import time
 import datetime
 import threading
+import urllib.parse
 
 import sabnzbd
 from sabnzbd.constants import RSS_FILE_NAME, DEFAULT_PRIORITY, DUP_PRIORITY
@@ -35,102 +36,21 @@ import sabnzbd.emailer as emailer
 
 import feedparser
 
-__RSS = None  # Global pointer to RSS-scanner instance
-
 
 ##############################################################################
 # Wrapper functions
 ##############################################################################
 
 
-def init():
-    global __RSS
-    __RSS = RSSQueue()
-
-
-def stop():
-    global __RSS
-    if __RSS:
-        __RSS.stop()
-        try:
-            __RSS.join()
-        except:
-            pass
-
-
-def run_feed(feed, download, ignoreFirst=False, force=False, readout=True):
-    global __RSS
-    if __RSS:
-        return __RSS.run_feed(feed, download, ignoreFirst, force=force, readout=readout)
-
-
-def show_result(feed):
-    global __RSS
-    if __RSS:
-        return __RSS.show_result(feed)
-
-
-def flag_downloaded(feed, fid):
-    global __RSS
-    if __RSS:
-        __RSS.flag_downloaded(feed, fid)
-
-
-def lookup_url(feed, fid):
-    global __RSS
-    if __RSS:
-        return __RSS.lookup_url(feed, fid)
-
-
-def run_method():
-    global __RSS
-    if __RSS:
-        return __RSS.run()
-    else:
-        return None
-
-
-def next_run(t=None):
-    global __RSS
-    if __RSS:
-        if t:
-            __RSS.next_run = t
-        else:
-            return __RSS.next_run
-    else:
-        return time.time()
-
-
-def save():
-    global __RSS
-    if __RSS:
-        __RSS.save()
-
-
-def clear_feed(feed):
-    global __RSS
-    if __RSS:
-        __RSS.clear_feed(feed)
-
-
-def clear_downloaded(feed):
-    global __RSS
-    if __RSS:
-        __RSS.clear_downloaded(feed)
-
-
-##############################################################################
-
-
 def notdefault(item):
     """ Return True if not 'Default|''|*' """
     return bool(item) and str(item).lower() not in ("default", "*", "", str(DEFAULT_PRIORITY))
 
 
 def convert_filter(text):
-    """ Return compiled regex.
-        If string starts with re: it's a real regex
-        else quote all regex specials, replace '*' by '.*'
+    """Return compiled regex.
+    If string starts with re: it's a real regex
+    else quote all regex specials, replace '*' by '.*'
     """
     text = text.strip().lower()
     if text.startswith("re:"):
@@ -145,13 +65,12 @@ def convert_filter(text):
 
 
 def remove_obsolete(jobs, new_jobs):
-    """ Expire G/B links that are not in new_jobs (mark them 'X')
-        Expired links older than 3 days are removed from 'jobs'
+    """Expire G/B links that are not in new_jobs (mark them 'X')
+    Expired links older than 3 days are removed from 'jobs'
     """
     now = time.time()
     limit = now - 259200  # 3days (3x24x3600)
-    olds = list(jobs.keys())
-    for old in olds:
+    for old in list(jobs):
         tm = jobs[old]["time"]
         if old not in new_jobs:
             if jobs[old].get("status", " ")[0] in ("G", "B"):
@@ -161,13 +80,13 @@ def remove_obsolete(jobs, new_jobs):
             del jobs[old]
 
 
-LOCK = threading.RLock()
+RSS_LOCK = threading.RLock()
 _RE_SP = re.compile(r"s*(\d+)[ex](\d+)", re.I)
 _RE_SIZE1 = re.compile(r"Size:\s*(\d+\.\d+\s*[KMG]{0,1})B\W*", re.I)
 _RE_SIZE2 = re.compile(r"\W*(\d+\.\d+\s*[KMG]{0,1})B\W*", re.I)
 
 
-class RSSQueue:
+class RSSReader:
     def __init__(self):
         self.jobs = {}
         self.next_run = time.time()
@@ -177,7 +96,7 @@ class RSSQueue:
             self.jobs = sabnzbd.load_admin(RSS_FILE_NAME)
             if self.jobs:
                 for feed in self.jobs:
-                    remove_obsolete(self.jobs[feed], list(self.jobs[feed].keys()))
+                    remove_obsolete(self.jobs[feed], list(self.jobs[feed]))
         except:
             logging.warning(T("Cannot read %s"), RSS_FILE_NAME)
             logging.info("Traceback: ", exc_info=True)
@@ -211,7 +130,7 @@ class RSSQueue:
     def stop(self):
         self.shutdown = True
 
-    @synchronized(LOCK)
+    @synchronized(RSS_LOCK)
     def run_feed(self, feed=None, download=False, ignoreFirst=False, force=False, readout=True):
         """ Run the query for one URI and apply filters """
         self.shutdown = False
@@ -232,7 +151,6 @@ class RSSQueue:
 
         uris = feeds.uri()
         defCat = feeds.cat()
-        import sabnzbd.api
 
         if not notdefault(defCat) or defCat not in sabnzbd.api.list_cats(default=False):
             defCat = None
@@ -277,44 +195,49 @@ class RSSQueue:
         feedparser.USER_AGENT = "SABnzbd/%s" % sabnzbd.__version__
 
         # Read the RSS feed
-        msg = None
-        entries = None
+        msg = ""
+        entries = []
         if readout:
             all_entries = []
             for uri in uris:
-                uri = uri.replace(" ", "%20")
+                # Reset parsing message for each feed
+                msg = ""
+                feed_parsed = {}
+                uri = uri.replace(" ", "%20").replace("feed://", "http://")
                 logging.debug("Running feedparser on %s", uri)
-                feed_parsed = feedparser.parse(uri.replace("feed://", "http://"))
-                logging.debug("Done parsing %s", uri)
-
-                if not feed_parsed:
-                    msg = T("Failed to retrieve RSS from %s: %s") % (uri, "?")
-                    logging.info(msg)
+                try:
+                    feed_parsed = feedparser.parse(uri)
+                except Exception as feedparser_exc:
+                    # Feedparser 5 would catch all errors, while 6 just throws them back at us
+                    feed_parsed["bozo_exception"] = feedparser_exc
+                logging.debug("Finished parsing %s", uri)
 
                 status = feed_parsed.get("status", 999)
                 if status in (401, 402, 403):
                     msg = T("Do not have valid authentication for feed %s") % uri
-                    logging.info(msg)
-
-                if 500 <= status <= 599:
+                elif 500 <= status <= 599:
                     msg = T("Server side error (server code %s); could not get %s on %s") % (status, feed, uri)
-                    logging.info(msg)
 
-                entries = feed_parsed.get("entries")
+                entries = feed_parsed.get("entries", [])
+                if not entries and "feed" in feed_parsed and "error" in feed_parsed["feed"]:
+                    msg = T("Failed to retrieve RSS from %s: %s") % (uri, feed_parsed["feed"]["error"])
+
+                # Exception was thrown
                 if "bozo_exception" in feed_parsed and not entries:
                     msg = str(feed_parsed["bozo_exception"])
                     if "CERTIFICATE_VERIFY_FAILED" in msg:
                         msg = T("Server %s uses an untrusted HTTPS certificate") % get_base_url(uri)
                         msg += " - https://sabnzbd.org/certificate-errors"
-                        logging.error(msg)
                     elif "href" in feed_parsed and feed_parsed["href"] != uri and "login" in feed_parsed["href"]:
                         # Redirect to login page!
                         msg = T("Do not have valid authentication for feed %s") % uri
                     else:
                         msg = T("Failed to retrieve RSS from %s: %s") % (uri, msg)
-                    logging.info(msg)
 
-                if not entries and not msg:
+                if msg:
+                    # We need to escape any "%20" that could be in the warning due to the URL's
+                    logging.warning_helpful(urllib.parse.unquote(msg))
+                elif not entries:
                     msg = T("RSS Feed %s was empty") % uri
                     logging.info(msg)
                 all_entries.extend(entries)
@@ -550,11 +473,11 @@ class RSSQueue:
         if not sabnzbd.PAUSED_ALL:
             active = False
             if self.next_run < time.time():
-                self.next_run = time.time() + cfg.rss_rate.get() * 60
+                self.next_run = time.time() + cfg.rss_rate() * 60
             feeds = config.get_rss()
             try:
                 for feed in feeds:
-                    if feeds[feed].enable.get():
+                    if feeds[feed].enable():
                         logging.info('Starting scheduled RSS read-out for "%s"', feed)
                         active = True
                         self.run_feed(feed, download=True, ignoreFirst=True)
@@ -573,7 +496,7 @@ class RSSQueue:
                 self.save()
                 logging.info("Finished scheduled RSS read-outs")
 
-    @synchronized(LOCK)
+    @synchronized(RSS_LOCK)
     def show_result(self, feed):
         if feed in self.jobs:
             try:
@@ -583,16 +506,22 @@ class RSSQueue:
         else:
             return {}
 
-    @synchronized(LOCK)
+    @synchronized(RSS_LOCK)
     def save(self):
         sabnzbd.save_admin(self.jobs, RSS_FILE_NAME)
 
-    @synchronized(LOCK)
+    @synchronized(RSS_LOCK)
     def delete(self, feed):
         if feed in self.jobs:
             del self.jobs[feed]
 
-    @synchronized(LOCK)
+    @synchronized(RSS_LOCK)
+    def rename(self, old_feed, new_feed):
+        if old_feed in self.jobs:
+            old_data = self.jobs.pop(old_feed)
+            self.jobs[new_feed] = old_data
+
+    @synchronized(RSS_LOCK)
     def flag_downloaded(self, feed, fid):
         if feed in self.jobs:
             lst = self.jobs[feed]
@@ -601,7 +530,7 @@ class RSSQueue:
                     lst[link]["status"] = "D"
                     lst[link]["time_downloaded"] = time.localtime()
 
-    @synchronized(LOCK)
+    @synchronized(RSS_LOCK)
     def lookup_url(self, feed, url):
         if url and feed in self.jobs:
             lst = self.jobs[feed]
@@ -610,13 +539,13 @@ class RSSQueue:
                     return lst[link]
         return None
 
-    @synchronized(LOCK)
+    @synchronized(RSS_LOCK)
     def clear_feed(self, feed):
         # Remove any previous references to this feed name, and start fresh
         if feed in self.jobs:
             del self.jobs[feed]
 
-    @synchronized(LOCK)
+    @synchronized(RSS_LOCK)
     def clear_downloaded(self, feed):
         # Mark downloaded jobs, so that they won't be displayed any more.
         if feed in self.jobs:
@@ -625,8 +554,8 @@ class RSSQueue:
                     self.jobs[feed][item]["status"] = "D-"
 
     def check_duplicate(self, title):
-        """ Check if this title was in this or other feeds
-            Return matching feed name
+        """Check if this title was in this or other feeds
+        Return matching feed name
         """
         title = title.lower()
         for fd in self.jobs:
@@ -638,18 +567,31 @@ class RSSQueue:
 
 
 def patch_feedparser():
-    """ Apply options that work for SABnzbd
-        Add additional parsing of attributes
+    """Apply options that work for SABnzbd
+    Add additional parsing of attributes
     """
     feedparser.SANITIZE_HTML = 0
-    feedparser.PARSE_MICROFORMATS = 0
+    feedparser.RESOLVE_RELATIVE_URIS = 0
+
+    # Support both feedparser 5 and 6
+    try:
+        feedparser_mixin = feedparser._FeedParserMixin
+        feedparser_parse_date = feedparser._parse_date
+    except AttributeError:
+        feedparser_mixin = feedparser.mixin._FeedParserMixin
+        feedparser_parse_date = feedparser.datetimes._parse_date
 
     # Add our own namespace
-    feedparser._FeedParserMixin.namespaces["http://www.newznab.com/DTD/2010/feeds/attributes/"] = "newznab"
+    feedparser_mixin.namespaces["http://www.newznab.com/DTD/2010/feeds/attributes/"] = "newznab"
 
     # Add parsers for the namespace
     def _start_newznab_attr(self, attrsD):
-        context = self._getContext()
+        # Support both feedparser 5 and 6
+        try:
+            context = self._getContext()
+        except AttributeError:
+            context = self._get_context()
+
         # Add the dict
         if "newznab" not in context:
             context["newznab"] = {}
@@ -659,14 +601,14 @@ def patch_feedparser():
             context["newznab"][attrsD["name"]] = attrsD["value"]
             # Try to get date-object
             if attrsD["name"] == "usenetdate":
-                context["newznab"][attrsD["name"] + "_parsed"] = feedparser._parse_date(attrsD["value"])
+                context["newznab"][attrsD["name"] + "_parsed"] = feedparser_parse_date(attrsD["value"])
         except KeyError:
             pass
 
-    feedparser._FeedParserMixin._start_newznab_attr = _start_newznab_attr
-    feedparser._FeedParserMixin._start_nZEDb_attr = _start_newznab_attr
-    feedparser._FeedParserMixin._start_nzedb_attr = _start_newznab_attr
-    feedparser._FeedParserMixin._start_nntmux_attr = _start_newznab_attr
+    feedparser_mixin._start_newznab_attr = _start_newznab_attr
+    feedparser_mixin._start_nZEDb_attr = _start_newznab_attr
+    feedparser_mixin._start_nzedb_attr = _start_newznab_attr
+    feedparser_mixin._start_nntmux_attr = _start_newznab_attr
 
 
 def _HandleLink(
@@ -728,8 +670,8 @@ def _HandleLink(
 
 
 def _get_link(entry):
-    """ Retrieve the post link from this entry
-        Returns (link, category, size)
+    """Retrieve the post link from this entry
+    Returns (link, category, size)
     """
     size = 0
     age = datetime.datetime.now()
@@ -809,8 +751,8 @@ def special_rss_site(url):
 
 
 def ep_match(season, episode, expr, title=None):
-    """ Return True if season, episode is at or above expected
-        Optionally `title` can be matched
+    """Return True if season, episode is at or above expected
+    Optionally `title` can be matched
     """
     m = _RE_SP.search(expr)
     if m:
