@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2020 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2021 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -42,8 +42,6 @@ from os.path import abspath
 import sabnzbd
 from sabnzbd.panic import launch_a_browser
 import sabnzbd.api as api
-import sabnzbd.scheduler as scheduler
-from sabnzbd.downloader import Downloader
 import sabnzbd.cfg as cfg
 from sabnzbd.misc import to_units
 
@@ -76,7 +74,7 @@ class StatusIcon(Thread):
         self.statusicon.set_visible(True)
         self.icon = self.sabicons["default"]
         self.refresh_icon()
-        self.tooltip = "SABnzbd"
+        self.tooltip = "SABnzbd %s" % sabnzbd.__version__
         self.refresh_tooltip()
         if HAVE_XAPP:
             self.statusicon.connect("activate", self.right_click_event)
@@ -113,6 +111,7 @@ class StatusIcon(Thread):
             self.icon = self.sabicons["default"]
 
         self.refresh_icon()
+        self.tooltip = "SABnzbd %s\n%s" % (sabnzbd.__version__, self.tooltip)
         self.refresh_tooltip()
         return 1
 
@@ -194,12 +193,12 @@ class StatusIcon(Thread):
         sabnzbd.shutdown_program()
 
     def pause(self):
-        scheduler.plan_resume(0)
-        Downloader.do.pause()
+        sabnzbd.Scheduler.plan_resume(0)
+        sabnzbd.Downloader.pause()
 
     def resume(self):
-        scheduler.plan_resume(0)
+        sabnzbd.Scheduler.plan_resume(0)
         sabnzbd.unpause_all()
 
     def rss(self, icon):
-        scheduler.force_rss()
+        sabnzbd.Scheduler.force_rss()
