@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2020 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2021 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -25,6 +25,7 @@ Generic Sorting - Sorting large files by a custom matching
 import os
 import logging
 import re
+from typing import Optional
 
 import sabnzbd
 from sabnzbd.filesystem import (
@@ -34,11 +35,13 @@ from sabnzbd.filesystem import (
     get_unique_filename,
     get_ext,
     renamer,
+    sanitize_and_trim_path,
     sanitize_foldername,
     clip_path,
 )
 from sabnzbd.constants import series_match, date_match, year_match, sample_match
 import sabnzbd.cfg as cfg
+from sabnzbd.nzbstuff import NzbObject
 
 RE_SAMPLE = re.compile(sample_match, re.I)
 # Do not rename .vob files as they are usually DVD's
@@ -147,7 +150,7 @@ def move_to_parent_folder(workdir):
 class Sorter:
     """ Generic Sorter class """
 
-    def __init__(self, nzo, cat):
+    def __init__(self, nzo: Optional[NzbObject], cat):
         self.sorter = None
         self.type = None
         self.sort_file = False
@@ -231,7 +234,7 @@ class Sorter:
 class SeriesSorter:
     """ Methods for Series Sorting """
 
-    def __init__(self, nzo, job_name, path, cat):
+    def __init__(self, nzo: Optional[NzbObject], job_name, path, cat):
         self.matched = False
 
         self.original_job_name = job_name
@@ -284,7 +287,8 @@ class SeriesSorter:
             # Error Sorting
             return os.path.join(self.original_path, self.original_job_name)
 
-    def get_multi_ep_naming(self, one, two, extras):
+    @staticmethod
+    def get_multi_ep_naming(one, two, extras):
         """ Returns a list of unique values joined into a string and separated by - (ex:01-02-03-04) """
         extra_list = [one]
         extra2_list = [two]
@@ -489,6 +493,7 @@ class SeriesSorter:
             newpath = os.path.join(current_path, newname)
             # Replace %ext with extension
             newpath = newpath.replace("%ext", self.ext)
+            newpath = sanitize_and_trim_path(newpath)
             try:
                 logging.debug("Rename: %s to %s", filepath, newpath)
                 renamer(filepath, newpath)
@@ -531,7 +536,7 @@ def check_for_sequence(regex, files):
                 prefix = name[: match1.start()]
 
     # Don't do anything if only one or no files matched
-    if len(list(matches.keys())) < 2:
+    if len(list(matches)) < 2:
         return {}
 
     key_prev = 0
@@ -539,7 +544,7 @@ def check_for_sequence(regex, files):
     alphabet = "abcdefghijklmnopqrstuvwxyz"
 
     # Check the dictionary to see if the keys are in a numeric or alphabetic sequence
-    for akey in sorted(matches.keys()):
+    for akey in sorted(matches):
         if akey.isdigit():
             key = int(akey)
         elif akey in alphabet:
@@ -569,7 +574,7 @@ def check_for_sequence(regex, files):
 class MovieSorter:
     """ Methods for Generic Sorting """
 
-    def __init__(self, nzo, job_name, path, cat):
+    def __init__(self, nzo: Optional[NzbObject], job_name, path, cat):
         self.matched = False
 
         self.original_job_name = job_name
@@ -783,7 +788,7 @@ class MovieSorter:
 class DateSorter:
     """ Methods for Date Sorting """
 
-    def __init__(self, nzo, job_name, path, cat):
+    def __init__(self, nzo: Optional[NzbObject], job_name, path, cat):
         self.matched = False
 
         self.original_job_name = job_name
@@ -978,10 +983,10 @@ class DateSorter:
 
 
 def path_subst(path, mapping):
-    """ Replace the sort sting elements by real values.
-        Non-elements are copied literally.
-        path = the sort string
-        mapping = array of tuples that maps all elements to their values
+    """Replace the sort sting elements by real values.
+    Non-elements are copied literally.
+    path = the sort string
+    mapping = array of tuples that maps all elements to their values
     """
     # Added ugly hack to prevent %ext from being masked by %e
     newpath = []
@@ -1000,12 +1005,12 @@ def path_subst(path, mapping):
     return "".join(newpath)
 
 
-def get_titles(nzo, match, name, titleing=False):
-    """ The title will be the part before the match
-        Clean it up and title() it
+def get_titles(nzo: NzbObject, match, name, titleing=False):
+    """The title will be the part before the match
+    Clean it up and title() it
 
-        ''.title() isn't very good under python so this contains
-        a lot of little hacks to make it better and for more control
+    ''.title() isn't very good under python so this contains
+    a lot of little hacks to make it better and for more control
     """
     if nzo:
         title = nzo.nzo_info.get("propername")
@@ -1081,15 +1086,15 @@ def replace_word(word_input, one, two):
     regex = re.compile(r"\W(%s)(\W|$)" % one, re.I)
     matches = regex.findall(word_input)
     if matches:
-        for unused in matches:
+        for _ in matches:
             word_input = word_input.replace(one, two)
     return word_input
 
 
-def get_descriptions(nzo, match, name):
-    """ If present, get a description from the nzb name.
-        A description has to be after the matched item, separated either
-        like ' - Description' or '_-_Description'
+def get_descriptions(nzo: NzbObject, match, name):
+    """If present, get a description from the nzb name.
+    A description has to be after the matched item, separated either
+    like ' - Description' or '_-_Description'
     """
     if nzo:
         ep_name = nzo.nzo_info.get("episodename")
@@ -1150,8 +1155,8 @@ def to_lowercase(path):
 
 
 def strip_folders(path):
-    """ Return 'path' without leading and trailing spaces and underscores in each element
-        For Windows, also remove leading and trailing dots
+    """Return 'path' without leading and trailing spaces and underscores in each element
+    For Windows, also remove leading and trailing dots
     """
     unc = sabnzbd.WIN32 and (path.startswith("//") or path.startswith("\\\\"))
     f = path.strip("/").split("/")
@@ -1164,7 +1169,7 @@ def strip_folders(path):
         """ Strip all leading/trailing underscores also dots for Windows """
         x = x.strip().strip("_")
         if sabnzbd.WIN32:
-            # OSX and Linux should keep dots, because leading dots are significant
+            # macOS and Linux should keep dots, because leading dots are significant
             # while Windows cannot handle trailing dots
             x = x.strip(".")
         x = x.strip()
@@ -1178,10 +1183,10 @@ def strip_folders(path):
 
 
 def rename_similar(folder, skip_ext, name, skipped_files):
-    """ Rename all other files in the 'folder' hierarchy after 'name'
-        and move them to the root of 'folder'.
-        Files having extension 'skip_ext' will be moved, but not renamed.
-        Don't touch files in list `skipped_files`
+    """Rename all other files in the 'folder' hierarchy after 'name'
+    and move them to the root of 'folder'.
+    Files having extension 'skip_ext' will be moved, but not renamed.
+    Don't touch files in list `skipped_files`
     """
     logging.debug('Give files in set "%s" matching names.', name)
     folder = os.path.normpath(folder)
@@ -1213,9 +1218,9 @@ def rename_similar(folder, skip_ext, name, skipped_files):
 
 
 def check_regexs(filename, matchers):
-    """ Regular Expression match for a list of regexes
-        Returns the MatchObject if a match is made
-        This version checks for an additional match
+    """Regular Expression match for a list of regexes
+    Returns the MatchObject if a match is made
+    This version checks for an additional match
     """
     extras = []
     for expressions in matchers:
@@ -1236,8 +1241,8 @@ def check_regexs(filename, matchers):
 
 
 def check_for_date(filename, matcher):
-    """ Regular Expression match for date based files
-        Returns the MatchObject if a match is made
+    """Regular Expression match for date based files
+    Returns the MatchObject if a match is made
     """
     x = 0
     if matcher:
