@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2020 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2021 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -221,5 +221,13 @@ if run_renamer:
     else:
         print("No par2 files or large files found")
 
+# Note about the new option
+print(
+    "The features of Deobfuscate.py are now integrated into SABnzbd! "
+    + "Just enable 'Deobfuscate final filenames' in Config - Switches. "
+    + "Don't forget to disable this script when you enable the new option!"
+    + "This script will be removed in the next version of SABnzbd."
+)
+
 # Always exit with success-code
 sys.exit(0)
